@@ -1,26 +1,20 @@
 #include "swoole.h"
 #include "server.h"
 
-
+namespace swoole_test
+{
 static int my_onPacket(swServer *serv, swEventData *req);
-
 static int my_onReceive(swServer *serv, swEventData *req);
-
 static void my_onStart(swServer *serv);
-
 static void my_onShutdown(swServer *serv);
-
 static void my_onConnect(swServer *serv, swDataHead *info);
-
 static void my_onClose(swServer *serv, swDataHead *info);
-
 static void my_onWorkerStart(swServer *serv, int worker_id);
-
 static void my_onWorkerStop(swServer *serv, int worker_id);
 
 static int g_receive_count = 0;
 
-int server_main(int argc, char **argv)
+int server_test()
 {
     int ret;
     swServer serv;
@@ -42,10 +36,19 @@ int server_main(int argc, char **argv)
 
 #ifdef HAVE_OPENSSL
     //serv.ssl_cert_file = "tests/ssl/ssl.crt";
-//serv.ssl_key_file = "tests/ssl/ssl.key";
-//serv.open_ssl = 1;
+    //serv.ssl_key_file = "tests/ssl/ssl.key";
+    //serv.open_ssl = 1;
 #endif
 
+    serv.onStart = my_onStart;
+    serv.onShutdown = my_onShutdown;
+    serv.onConnect = my_onConnect;
+    serv.onReceive = my_onReceive;
+    serv.onPacket = my_onPacket;
+    serv.onClose = my_onClose;
+    serv.onWorkerStart = my_onWorkerStart;
+    serv.onWorkerStop = my_onWorkerStop;
+
 //	swSignal_add(SIGINT, user_signal);
 
     //create Server
@@ -56,24 +59,15 @@ int server_main(int argc, char **argv)
         exit(0);
     }
 
-    serv.onStart = my_onStart;
-    serv.onShutdown = my_onShutdown;
-    serv.onConnect = my_onConnect;
-    serv.onReceive = my_onReceive;
-    serv.onPacket = my_onPacket;
-    serv.onClose = my_onClose;
-    serv.onWorkerStart = my_onWorkerStart;
-    serv.onWorkerStop = my_onWorkerStop;
-
-    swListenPort *port = swServer_add_port(&serv, SW_SOCK_TCP, (char *) "127.0.0.1", 9501);
+    swListenPort *port = swServer_add_port(&serv, SW_SOCK_TCP, "127.0.0.1", 9501);
     port->open_eof_check = 0;
     //config
     port->backlog = 128;
     memcpy(port->protocol.package_eof, SW_STRL("\r\n\r\n"));  //开启eof检测，启用buffer区
 
-    swServer_add_port(&serv, SW_SOCK_UDP, (char *) "0.0.0.0", 9502);
-    swServer_add_port(&serv, SW_SOCK_TCP6, (char *) "::", 9503);
-    swServer_add_port(&serv, SW_SOCK_UDP6, (char *) "::", 9504);
+    swServer_add_port(&serv, SW_SOCK_UDP, "0.0.0.0", 9502);
+    swServer_add_port(&serv, SW_SOCK_TCP6, "::", 9503);
+    swServer_add_port(&serv, SW_SOCK_UDP6, "::", 9504);
 
     ret = swServer_start(&serv);
     if (ret < 0)
@@ -97,16 +91,16 @@ void my_onWorkerStop(swServer *serv, int worker_id)
 int my_onReceive(swServer *serv, swEventData *req)
 {
     int ret;
-    char resp_data[SW_BUFFER_SIZE];
+    char resp_data[SW_IPC_BUFFER_SIZE];
 
     g_receive_count++;
 
     swConnection *conn = swWorker_get_connection(serv, req->info.fd);
     swoole_rtrim(req->data, req->info.len);
     printf("onReceive[%d]: ip=%s|port=%d Data=%s|Len=%d\n", g_receive_count, swConnection_get_ip(conn),
-           swConnection_get_port(conn), req->data, req->info.len);
+            swConnection_get_port(conn), req->data, req->info.len);
 
-    int n = snprintf(resp_data, SW_BUFFER_SIZE, "Server: %*s\n", req->info.len, req->data);
+    int n = snprintf(resp_data, SW_IPC_BUFFER_SIZE, "Server: %*s\n", req->info.len, req->data);
     ret = serv->send(serv, req->info.fd, resp_data, n);
     if (ret < 0)
     {
@@ -121,13 +115,11 @@ int my_onReceive(swServer *serv, swEventData *req)
 
 int my_onPacket(swServer *serv, swEventData *req)
 {
-    swDgramPacket *packet;
-
-    swString *buffer = swWorker_get_buffer(serv, req->info.from_id);
-    packet = (swDgramPacket *) buffer->str;
-
     int serv_sock = req->info.from_fd;
     char *data;
+    swWorker_get_data(req, &data);
+    swDgramPacket *packet = (swDgramPacket *) data;
+
     int length;
     char address[256];
     int port = 0;
@@ -136,51 +128,46 @@ int my_onPacket(swServer *serv, swEventData *req)
     //udp ipv4
     if (req->info.type == SW_EVENT_UDP)
     {
-        struct in_addr sin_addr;
-        sin_addr.s_addr = packet->addr.v4.s_addr;
-        char *tmp = inet_ntoa(sin_addr);
-        memcpy(address, tmp, strlen(tmp));
+        inet_ntop(AF_INET6, &packet->info.addr.inet_v4.sin_addr, address, sizeof(address));
         data = packet->data;
         length = packet->length;
-        port = packet->port;
+        port = ntohs(packet->info.addr.inet_v4.sin_port);
     }
-        //udp ipv6
+    //udp ipv6
     else if (req->info.type == SW_EVENT_UDP6)
     {
-        inet_ntop(AF_INET6, &packet->addr.v6, address, sizeof(address));
+        inet_ntop(AF_INET6, &packet->info.addr.inet_v6.sin6_addr, address, sizeof(address));
         data = packet->data;
         length = packet->length;
-        port = packet->port;
+        port = ntohs(packet->info.addr.inet_v6.sin6_port);
     }
-        //unix dgram
+    //unix dgram
     else if (req->info.type == SW_EVENT_UNIX_DGRAM)
     {
-        memcpy(address, packet->data, packet->addr.un.path_length);
-        data = packet->data + packet->addr.un.path_length;
-        length = packet->length - packet->addr.un.path_length;
+        strcpy(address, packet->info.addr.un.sun_path);
+        data = packet->data;
+        length = packet->length;
     }
 
     printf("Packet[client=%s:%d, %d bytes]: data=%*s\n", address, port, length, length, data);
 
-    char resp_data[SW_BUFFER_SIZE];
-    int n = snprintf(resp_data, SW_BUFFER_SIZE, "Server: %*s", length, data);
+    char resp_data[SW_IPC_BUFFER_SIZE];
+    int n = snprintf(resp_data, SW_IPC_BUFFER_SIZE, "Server: %*s", length, data);
 
     //udp ipv4
     if (req->info.type == SW_EVENT_UDP)
     {
         ret = swSocket_udp_sendto(serv_sock, address, port, resp_data, n);
     }
-        //udp ipv6
+    //udp ipv6
     else if (req->info.type == SW_EVENT_UDP6)
     {
         ret = swSocket_udp_sendto6(serv_sock, address, port, resp_data, n);
     }
-        //unix dgram
+    //unix dgram
     else if (req->info.type == SW_EVENT_UNIX_DGRAM)
     {
-        memcpy(address, packet->data, packet->addr.un.path_length);
-        data = packet->data + packet->addr.un.path_length;
-        length = packet->length - packet->addr.un.path_length;
+        ret = swSocket_unix_sendto(serv_sock, address, resp_data, n);
     }
 
     if (ret < 0)
@@ -214,3 +201,5 @@ void my_onClose(swServer *serv, swDataHead *info)
 {
     printf("PID=%d\tClose fd=%d|from_id=%d\n", getpid(), info->fd, info->from_id);
 }
+
+}
