@@ -1,18 +1,20 @@
 <?php
-//$serv = new swoole_server("0.0.0.0", 9501, SWOOLE_BASE, SWOOLE_SOCK_TCP | SWOOLE_SSL);
-$serv = new swoole_server("0.0.0.0", 9501, SWOOLE_PROCESS, SWOOLE_SOCK_TCP | SWOOLE_SSL);
+$serv = new swoole_server("0.0.0.0", 9501, SWOOLE_BASE, SWOOLE_SOCK_TCP | SWOOLE_SSL);
+// $serv = new swoole_server("0.0.0.0", 9501, SWOOLE_PROCESS, SWOOLE_SOCK_TCP | SWOOLE_SSL);
 $key_dir = dirname(dirname(__DIR__)).'/tests/ssl';
 
-$port2 = $serv->addlistener('0.0.0.0', 9502, SWOOLE_SOCK_TCP);
-$port2->on('receive', function($serv, $fd, $from_id, $data){
-    echo "port2: ".$data."\n";
-});
+// $port2 = $serv->addlistener('0.0.0.0', 9502, SWOOLE_SOCK_TCP);
+// $port2->on('receive', function($serv, $fd, $from_id, $data){
+//     echo "port2: ".$data."\n";
+// });
 
 $serv->set(array(
 //	'worker_num' => 4,
-	'ssl_cert_file' => __DIR__.'/corpssl.crt',
-	'ssl_key_file' => __DIR__.'/corpssl.key',
-    'ssl_client_cert_file' => __DIR__.'/ca.crt',
+	'ssl_cert_file' => __DIR__.'/ca/server-cert.pem',
+	'ssl_key_file' => __DIR__.'/ca/server-key.pem',
+    'ssl_verify_peer' => true,
+    'ssl_allow_self_signed' => true,
+    'ssl_client_cert_file' => __DIR__.'/ca/ca-cert.pem',
     'ssl_verify_depth' => 10,
 ));
 
