@@ -19,6 +19,11 @@
 
 enum swErrorCode
 {
+    /**
+     * Prevent repetition with errno [syscall error]
+     */
+    SW_ERROR_START = 500,
+
     /**
      * common error
      */
@@ -110,6 +115,7 @@ enum swErrorCode
     SW_ERROR_SERVER_ONLY_START_ONE,
     SW_ERROR_SERVER_SEND_IN_MASTER,
     SW_ERROR_SERVER_INVALID_REQUEST,
+    SW_ERROR_SERVER_CONNECT_FAIL,
 
     /**
      * Process exit timeout, forced to end.
@@ -134,9 +140,11 @@ enum swErrorCode
     SW_ERROR_CO_PROTECT_STACK_FAILED,
     SW_ERROR_CO_STD_THREAD_LINK_ERROR,
     SW_ERROR_CO_DISABLED_MULTI_THREAD,
+
+    SW_ERROR_END
 };
 
-const char* swstrerror(enum swErrorCode code);
+const char* swoole_strerror(enum swErrorCode code);
 void swoole_throw_error(enum swErrorCode code);
 
 #endif /* SW_ERRNO_H_ */
