@@ -32,37 +32,32 @@ extern "C" {
 #define SW_REACTOR_NUM             SW_CPU_NUM
 #define SW_WORKER_NUM              (SW_CPU_NUM*2)
 
-#define SW_HEARTBEAT_IDLE          0   //心跳存活最大时间
-#define SW_HEARTBEAT_CHECK         0   //心跳定时侦测时间
-
-enum swEventType
+enum swServer_event_type
 {
     //networking socket
-    SW_EVENT_TCP             = 0,
-    SW_EVENT_UDP             = 1,
-    SW_EVENT_TCP6            = 2,
-    SW_EVENT_UDP6            = 3,
+    SW_EVENT_TCP,
+    SW_EVENT_UDP,
+    SW_EVENT_TCP6,
+    SW_EVENT_UDP6,
     //tcp event
-    SW_EVENT_CLOSE           = 4,
-    SW_EVENT_CONNECT         = 5,
+    SW_EVENT_CLOSE,
+    SW_EVENT_CONNECT,
     //timer
-    SW_EVENT_TIMER           = 6,
+    SW_EVENT_TIMER,
     //task
-    SW_EVENT_TASK            = 7,
-    SW_EVENT_FINISH          = 8,
-    //package
-    SW_EVENT_PACKAGE_START   = 9,
-    SW_EVENT_PACKAGE_END     = 10,
-    SW_EVENT_PACKAGE         = 11,
-    SW_EVENT_SENDFILE        = 12,
-    SW_EVENT_UNIX_DGRAM      = 13,
-    SW_EVENT_UNIX_STREAM     = 14,
+    SW_EVENT_TASK,
+    SW_EVENT_FINISH,
+    //sendfile
+    SW_EVENT_SENDFILE,
+    //dgram
+    SW_EVENT_UNIX_DGRAM,
+    SW_EVENT_UNIX_STREAM,
     //pipe
-    SW_EVENT_PIPE_MESSAGE    = 15,
+    SW_EVENT_PIPE_MESSAGE,
     //proxy
-    SW_EVENT_PROXY_START     = 16,
-    SW_EVENT_PROXY_END       = 17,
-    SW_EVENT_CONFIRM         = 18,
+    SW_EVENT_PROXY_START,
+    SW_EVENT_PROXY_END,
+    SW_EVENT_CONFIRM,
     //event operate
     SW_EVENT_PAUSE_RECV,
     SW_EVENT_RESUME_RECV,
@@ -114,14 +109,13 @@ enum swTaskType
     SW_TASK_WAITALL    = 16, //for taskWaitAll
     SW_TASK_COROUTINE  = 32, //coroutine
     SW_TASK_PEEK       = 64, //peek
+    SW_TASK_NOREPLY    = 128, //don't reply
 };
 
 typedef struct _swReactorThread
 {
     pthread_t thread_id;
     swReactor reactor;
-    swMemoryPool *buffer_input;
-    swLock lock;
     int notify_pipe;
 } swReactorThread;
 
@@ -259,9 +253,9 @@ typedef struct
 //-----------------------------------Factory--------------------------------------------
 typedef struct
 {
-    long target_worker_id;
-    swEventData data;
-} swDispatchData;
+    swDataHead info;
+    swString data;
+} swPackagePtr;
 
 struct _swFactory
 {
@@ -273,7 +267,7 @@ struct _swFactory
 
     int (*start)(struct _swFactory *);
     int (*shutdown)(struct _swFactory *);
-    int (*dispatch)(struct _swFactory *, swDispatchData *);
+    int (*dispatch)(struct _swFactory *, swSendData *);
     int (*finish)(struct _swFactory *, swSendData *);
     int (*notify)(struct _swFactory *, swDataHead *);    //send a event notify
     int (*end)(struct _swFactory *, int fd);
@@ -284,12 +278,12 @@ typedef struct _swFactoryProcess
     swPipe *pipes;
 } swFactoryProcess;
 
-typedef int (*swServer_dispatch_function)(swServer *, swConnection *, swEventData *);
+typedef int (*swServer_dispatch_function)(swServer *, swConnection *, swSendData *);
 
 int swFactory_create(swFactory *factory);
 int swFactory_start(swFactory *factory);
 int swFactory_shutdown(swFactory *factory);
-int swFactory_dispatch(swFactory *factory, swDispatchData *req);
+int swFactory_dispatch(swFactory *factory, swSendData *req);
 int swFactory_finish(swFactory *factory, swSendData *_send);
 int swFactory_notify(swFactory *factory, swDataHead *event);
 int swFactory_end(swFactory *factory, int fd);
@@ -393,9 +387,6 @@ struct _swServer
      */
     uint32_t max_request;
 
-    int signal_fd;
-    int event_fd;
-
     int udp_socket_ipv4;
     int udp_socket_ipv6;
 
@@ -472,7 +463,10 @@ struct _swServer
      * enable coroutine
      */
     uint32_t enable_coroutine :1;
-
+    /**
+     * disable multi-threads
+     */
+    uint32_t single_thread :1;
     /**
      *  heartbeat check time
      */
@@ -495,7 +489,6 @@ struct _swServer
     void *ptr2;
     void *private_data_3;
 
-    swReactor reactor;
     swFactory factory;
     swListenPort *listen_list;
     pthread_t heartbeat_pidt;
@@ -536,29 +529,26 @@ struct _swServer
      * temporary directory for HTTP uploaded file.
      */
     char *upload_tmp_dir;
-
     /**
      * http compression level for gzip/br
      */
     uint8_t http_compression_level;
-
     /**
      * http static file directory
      */
     char *document_root;
     uint16_t document_root_len;
-
     /**
      * master process pid
      */
     char *pid_file;
-
     /**
      * stream
      */
     char *stream_socket;
     int stream_fd;
     swProtocol stream_protocol;
+    int last_stream_fd;
     swLinkedList *buffer_pool;
 
 #ifdef SW_BUFFER_RECV_TIME
@@ -571,7 +561,6 @@ struct _swServer
      * message queue key
      */
     uint64_t message_queue_key;
-
     /**
      * slow request log
      */
@@ -607,12 +596,17 @@ struct _swServer
      */
     int (*onTask)(swServer *serv, swEventData *data);
     int (*onFinish)(swServer *serv, swEventData *data);
+    /**
+     * Server method
+     */
+    int (*send)(swServer *serv, int session_id, void *data, uint32_t length);
+    int (*sendfile)(swServer *serv, int session_id, char *file, uint32_t l_file, off_t offset, size_t length);
+    int (*sendwait)(swServer *serv, int session_id, void *data, uint32_t length);
+    int (*close)(swServer *serv, int session_id, int reset);
+    int (*notify)(swServer *serv, swConnection *conn, int event);
+    int (*feedback)(swServer *serv, int session_id, int event);
 
-    int (*send)(swServer *serv, int fd, void *data, uint32_t length);
-    int (*sendfile)(swServer *serv, int fd, char *filename, uint32_t filename_length, off_t offset, size_t length);
-    int (*sendwait)(swServer *serv, int fd, void *data, uint32_t length);
-    int (*close)(swServer *serv, int fd, int reset);
-    int (*dispatch_func)(swServer *, swConnection *, swEventData *);
+    int (*dispatch_func)(swServer *, swConnection *, swSendData *);
 };
 
 typedef struct
@@ -681,12 +675,6 @@ static sw_inline swListenPort* swServer_get_port(swServer *serv, int fd)
 }
 
 int swServer_udp_send(swServer *serv, swSendData *resp);
-int swServer_tcp_send(swServer *serv, int fd, void *data, uint32_t length);
-int swServer_tcp_sendwait(swServer *serv, int fd, void *data, uint32_t length);
-int swServer_tcp_close(swServer *serv, int fd, int reset);
-int swServer_tcp_sendfile(swServer *serv, int session_id, char *filename, uint32_t filename_length, off_t offset, size_t length);
-int swServer_tcp_notify(swServer *serv, swConnection *conn, int event);
-int swServer_tcp_feedback(swServer *serv, int fd, int event);
 
 #define SW_MAX_SESSION_ID             0x1000000
 
@@ -710,9 +698,6 @@ static sw_inline int swEventData_is_stream(uint8_t type)
     case SW_EVENT_TCP:
     case SW_EVENT_TCP6:
     case SW_EVENT_UNIX_STREAM:
-    case SW_EVENT_PACKAGE_START:
-    case SW_EVENT_PACKAGE:
-    case SW_EVENT_PACKAGE_END:
     case SW_EVENT_CONNECT:
     case SW_EVENT_CLOSE:
     case SW_EVENT_PAUSE_RECV:
@@ -729,11 +714,11 @@ swPipe * swServer_get_pipe_object(swServer *serv, int pipe_fd);
 void swServer_store_pipe_fd(swServer *serv, swPipe *p);
 void swServer_store_listen_socket(swServer *serv);
 
-int swServer_get_manager_pid(swServer *serv);
 int swServer_get_socket(swServer *serv, int port);
 int swServer_worker_create(swServer *serv, swWorker *worker);
 int swServer_worker_init(swServer *serv, swWorker *worker);
 void swServer_worker_start(swServer *serv, swWorker *worker);
+
 swString** swServer_create_worker_buffer(swServer *serv);
 int swServer_create_task_worker(swServer *serv);
 void swServer_enable_accept(swReactor *reactor);
@@ -842,7 +827,7 @@ static sw_inline swWorker* swServer_get_worker(swServer *serv, uint16_t worker_i
     return NULL;
 }
 
-static sw_inline int swServer_worker_schedule(swServer *serv, int fd, swEventData *data)
+static sw_inline int swServer_worker_schedule(swServer *serv, int fd, swSendData *data)
 {
     uint32_t key;
 
@@ -948,6 +933,29 @@ static sw_inline swString *swWorker_get_buffer(swServer *serv, int reactor_id)
     }
 }
 
+static sw_inline size_t swWorker_get_data(swEventData *req, char **data_ptr)
+{
+    size_t length;
+    if (req->info.flags & SW_EVENT_DATA_PTR)
+    {
+        swPackagePtr *task = (swPackagePtr *) req;
+        *data_ptr = task->data.str;
+        length = task->data.length;
+    }
+    else if (req->info.flags & SW_EVENT_DATA_END)
+    {
+        swString *worker_buffer = swWorker_get_buffer(SwooleG.serv, req->info.from_id);
+        *data_ptr = worker_buffer->str;
+        length = worker_buffer->length;
+    }
+    else
+    {
+        *data_ptr = req->data;
+        length = req->info.len;
+    }
+    return length;
+}
+
 static sw_inline swConnection *swServer_connection_verify_no_ssl(swServer *serv, uint32_t session_id)
 {
     swSession *session = swServer_get_session(serv, session_id);
