@@ -94,6 +94,7 @@ extern zend_module_entry swoole_module_entry;
 #endif
 
 #define SWOOLE_PROPERTY_MAX     32
+#define SWOOLE_OBJECT_DEFAULT   65536
 #define SWOOLE_OBJECT_MAX       10000000
 
 typedef struct
@@ -139,7 +140,7 @@ extern swoole_object_array swoole_objects;
 #endif
 #endif
 
-#if PHP_VERSION_ID < 70300
+#if PHP_VERSION_ID < 70400
 #define SW_USE_FAST_SERIALIZE 1
 #endif
 
@@ -249,7 +250,7 @@ typedef struct
 {
     zend_fcall_info fci;
     zend_fcall_info_cache fci_cache;
-} php_defer_fci;
+} php_swoole_fci;
 //---------------------------------------------------------
 #define php_swoole_socktype(type)           (type & (~SW_FLAG_SYNC) & (~SW_FLAG_ASYNC) & (~SW_FLAG_KEEP) & (~SW_SOCK_SSL))
 
@@ -328,6 +329,11 @@ PHP_FUNCTION(swoole_timer_clear);
 //---------------------------------------------------------
 //                  error
 //---------------------------------------------------------
+#define SW_STRERROR_SYSTEM  0
+#define SW_STRERROR_GAI     1
+#define SW_STRERROR_DNS     2
+#define SW_STRERROR_SWOOLE  9
+
 PHP_FUNCTION(swoole_strerror);
 PHP_FUNCTION(swoole_errno);
 PHP_FUNCTION(swoole_last_error);
@@ -400,7 +406,6 @@ void php_swoole_register_shutdown_function_prepend(char *function);
 void php_swoole_event_init();
 void php_swoole_event_wait();
 void php_swoole_event_exit();
-long php_swoole_add_timer(long ms, zval *callback, zval *param, int persistent);
 void php_swoole_clear_all_timer();
 void php_swoole_register_callback(swServer *serv);
 void php_swoole_trace_check(void *arg);
@@ -480,10 +485,8 @@ void php_swoole_onClose(swServer *, swDataHead *);
 void php_swoole_onBufferFull(swServer *, swDataHead *);
 void php_swoole_onBufferEmpty(swServer *, swDataHead *);
 ssize_t php_swoole_length_func(swProtocol *protocol, swConnection *conn, char *data, uint32_t length);
-int php_swoole_dispatch_func(swServer *serv, swConnection *conn, swEventData *data);
+int php_swoole_dispatch_func(swServer *serv, swConnection *conn, swSendData *data);
 int php_swoole_client_onPackage(swConnection *conn, char *data, uint32_t length);
-void php_swoole_onTimeout(swTimer *timer, swTimer_node *tnode);
-void php_swoole_onInterval(swTimer *timer, swTimer_node *tnode);
 zend_bool php_swoole_signal_isset_handler(int signo);
 void php_swoole_event_onDefer(void *_cb);
 
@@ -614,7 +617,7 @@ static sw_inline void _sw_zend_bailout(const char *filename, uint32_t lineno)
 
 #ifndef GC_IS_RECURSIVE
 #define GC_IS_RECURSIVE(p) \
-	(ZEND_HASH_GET_APPLY_COUNT(p) > 1)
+	(ZEND_HASH_GET_APPLY_COUNT(p) >= 1)
 #define GC_PROTECT_RECURSION(p) \
 	ZEND_HASH_INC_APPLY_COUNT(p)
 #define GC_UNPROTECT_RECURSION(p) \
@@ -691,7 +694,7 @@ static sw_inline void sw_zval_free(zval *val)
 #define SW_HASHTABLE_FOREACH_START2(ht, k, klen, ktype, _val) zend_string *_foreach_key;\
     ZEND_HASH_FOREACH_STR_KEY_VAL(ht, _foreach_key, _val); \
     if (!_foreach_key) {k = NULL; klen = 0; ktype = 0;} \
-    else {k = _foreach_key->val, klen=_foreach_key->len; ktype = 1;} {
+    else {k = ZSTR_VAL(_foreach_key), klen=ZSTR_LEN(_foreach_key); ktype = 1;} {
 #define SW_HASHTABLE_FOREACH_END()                 } ZEND_HASH_FOREACH_END();
 
 static sw_inline int add_assoc_ulong_safe(zval *arg, const char *key, zend_ulong value)
@@ -751,10 +754,16 @@ static sw_inline int add_assoc_ulong_safe(zval *arg, const char *key, zend_ulong
 #define SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(module, _unset_property) \
     module##_handlers.unset_property = _unset_property;
 
-#define SWOOLE_SET_CLASS_CREATE_AND_FREE(module, _create_object, _free_obj) \
-    module##_ce_ptr->create_object = _create_object; \
+#define SWOOLE_SET_CLASS_CREATE(module, _create_object) \
+    module##_ce_ptr->create_object = _create_object;
+
+#define SWOOLE_SET_CLASS_FREE(module, _free_obj) \
     module##_handlers.free_obj = _free_obj;
 
+#define SWOOLE_SET_CLASS_CREATE_AND_FREE(module, _create_object, _free_obj) \
+    SWOOLE_SET_CLASS_CREATE(module, _create_object); \
+    SWOOLE_SET_CLASS_FREE(module, _free_obj);
+
 #define SWOOLE_SET_CLASS_CUSTOM_OBJECT(module, _create_object, _free_obj, _struct, _std) \
     SWOOLE_SET_CLASS_CREATE_AND_FREE(module, _create_object, _free_obj); \
     module##_handlers.offset = XtOffsetOf(_struct, _std);
@@ -778,9 +787,9 @@ static sw_inline int sw_zend_register_class_alias(const char *name, size_t name_
     zend_string *_interned_name = zend_new_interned_string(_name);
 
 #if PHP_VERSION_ID >= 70300
-    return zend_register_class_alias_ex(_interned_name->val, _interned_name->len, ce, 1);
+    return zend_register_class_alias_ex(ZSTR_VAL(_interned_name), ZSTR_LEN(_interned_name), ce, 1);
 #else
-    return zend_register_class_alias_ex(_interned_name->val, _interned_name->len, ce);
+    return zend_register_class_alias_ex(ZSTR_VAL(_interned_name), ZSTR_LEN(_interned_name), ce);
 #endif
 }
 
@@ -840,13 +849,19 @@ static sw_inline zval* sw_zend_read_property_array(zend_class_entry *class_ptr,
     if (ZVAL_IS_NULL(&__retval)) *(retval) = NULL;\
     else *(retval) = &__retval;
 
-// TODO: remove it after remove async modules
+static sw_inline int sw_zend_function_max_num_args(zend_function *function)
+{
+    // https://github.com/php/php-src/commit/2646f7bcb98dcdd322ea21701c8bb101104ea619
+    // zend_function.common.num_args don't include the variadic argument anymore.
+    return (function->common.fn_flags & ZEND_ACC_VARIADIC) ? UINT32_MAX : function->common.num_args;
+}
 
+// TODO: remove it after remove async modules
 static sw_inline int sw_zend_is_callable(zval *cb, int a, char **name)
 {
     zend_string *key = NULL;
     int ret = zend_is_callable(cb, a, &key);
-    char *tmp = estrndup(key->val, key->len);
+    char *tmp = estrndup(ZSTR_VAL(key), ZSTR_LEN(key));
     zend_string_release(key);
     *name = tmp;
     return ret;
@@ -876,7 +891,7 @@ static sw_inline int sw_zend_is_callable_ex(zval *zcallable, zval *zobject, uint
 {
     zend_string *key = NULL;
     int ret = zend_is_callable_ex(zcallable, NULL, check_flags, &key, fci_cache, error);
-    char *tmp = estrndup(key->val, key->len);
+    char *tmp = estrndup(ZSTR_VAL(key), ZSTR_LEN(key));
     zend_string_release(key);
     *callable_name = tmp;
     return ret;
@@ -931,6 +946,33 @@ static sw_inline int sw_call_function_anyway(zend_fcall_info *fci, zend_fcall_in
     return ret;
 }
 
+static sw_inline void sw_fci_params_persist(zend_fcall_info *fci)
+{
+    if (fci->param_count > 0)
+    {
+        uint32_t i;
+        zval *params = (zval *) ecalloc(fci->param_count, sizeof(zval));
+        for (i = 0; i < fci->param_count; i++)
+        {
+            ZVAL_COPY(&params[i], &fci->params[i]);
+        }
+        fci->params = params;
+    }
+}
+
+static sw_inline void sw_fci_params_discard(zend_fcall_info *fci)
+{
+    if (fci->param_count > 0)
+    {
+        uint32_t i;
+        for (i = 0; i < fci->param_count; i++)
+        {
+            zval_ptr_dtor(&fci->params[i]);
+        }
+        efree(fci->params);
+    }
+}
+
 static sw_inline void sw_fci_cache_persist(zend_fcall_info_cache *fci_cache)
 {
     if (fci_cache->object)
@@ -960,7 +1002,7 @@ static sw_inline void sw_fci_cache_discard(zend_fcall_info_cache *fci_cache)
 static sw_inline char* sw_php_format_date(char *format, size_t format_len, time_t ts, int localtime)
 {
     zend_string *time = php_format_date(format, format_len, ts, localtime);
-    char *return_str = estrndup(time->val, time->len);
+    char *return_str = estrndup(ZSTR_VAL(time), ZSTR_LEN(time));
     zend_string_release(time);
     return return_str;
 }
@@ -968,8 +1010,8 @@ static sw_inline char* sw_php_format_date(char *format, size_t format_len, time_
 static sw_inline char* sw_php_url_encode(char *value, size_t value_len, int* exten)
 {
     zend_string *str = php_url_encode(value, value_len);
-    *exten = str->len;
-    char *return_str = estrndup(str->val, str->len);
+    *exten = ZSTR_LEN(str);
+    char *return_str = estrndup(ZSTR_VAL(str), ZSTR_LEN(str));
     zend_string_release(str);
     return return_str;
 }
