@@ -108,6 +108,8 @@ void swoole_init(void)
     }
 #endif
 
+    SwooleG.socket_buffer_size = SW_SOCKET_BUFFER_SIZE;
+
     SwooleTG.buffer_stack = swString_new(SW_STACK_BUFFER_SIZE);
     if (SwooleTG.buffer_stack == NULL)
     {
@@ -231,7 +233,7 @@ int swoole_mkdir_recursive(const char *dir)
         swWarn("mkdir(%s) failed. Path exceeds the limit of %d characters.", dir, PATH_MAX - 1);
         return -1;
     }
-    strncpy(tmp, dir, len + 1);
+    strncpy(tmp, dir, PATH_MAX);
 
     if (dir[len - 1] != '/')
     {
@@ -774,6 +776,27 @@ void swBreakPoint()
 
 }
 
+size_t sw_snprintf(char *buf, size_t size, const char *format, ...)
+{
+    int ret;
+    va_list args;
+
+    va_start(args, format);
+    ret = vsnprintf(buf, size, format, args);
+    va_end(args);
+    if (unlikely(ret < 0))
+    {
+        ret = 0;
+        buf[0] = '\0';
+    }
+    else if (unlikely(ret >= size))
+    {
+        ret = size - 1;
+        buf[ret] = '\0';
+    }
+    return ret;
+}
+
 void swoole_ioctl_set_block(int sock, int nonblock)
 {
     int ret;
@@ -1204,7 +1227,7 @@ SW_API void swoole_call_hook(enum swGlobal_hook_type type, void *arg)
     }
 }
 
-int swoole_shell_exec(char *command, pid_t *pid, uint8_t get_error_stream)
+int swoole_shell_exec(const char *command, pid_t *pid, uint8_t get_error_stream)
 {
     pid_t child_pid;
     int fds[2];
@@ -1265,21 +1288,20 @@ int swoole_shell_exec(char *command, pid_t *pid, uint8_t get_error_stream)
 char* swoole_string_format(size_t n, const char *format, ...)
 {
     char *buf = sw_malloc(n);
-    if (buf == NULL)
-    {
-        return NULL;
-    }
-
-    va_list _va_list;
-    va_start(_va_list, format);
-
-    if (vsnprintf(buf, n, format, _va_list) < 0)
+    if (buf)
     {
+        int ret;
+        va_list va_list;
+        va_start(va_list, format);
+        ret = vsnprintf(buf, n, format, va_list);
+        va_end(va_list);
+        if (ret >= 0)
+        {
+            return buf;
+        }
         sw_free(buf);
-        return NULL;
     }
-
-    return buf;
+    return NULL;
 }
 
 #ifdef HAVE_EXECINFO
