@@ -13,22 +13,181 @@ using namespace std;
 
 void Socket::timer_callback(swTimer *timer, swTimer_node *tnode)
 {
-    Socket *sock = (Socket *) tnode->data;
-    swTraceLog(SW_TRACE_SOCKET, "socket[%d] timeout", sock->socket->fd);
-    sock->set_err(ETIMEDOUT);
-    sock->reactor->del(sock->reactor, sock->socket->fd);
-    sock->timer = NULL;
-    sock->resume();
+    Socket *socket = (Socket *) tnode->data;
+    socket->set_err(ETIMEDOUT);
+    if (likely(tnode == socket->read_timer))
+    {
+        socket->read_timer = nullptr;
+        socket->read_co->resume();
+    }
+    else if (tnode == socket->write_timer)
+    {
+        socket->write_timer = nullptr;
+        socket->write_co->resume();
+    }
+    else
+    {
+        assert(0);
+    }
 }
 
-int Socket::event_callback(swReactor *reactor, swEvent *event)
+int Socket::readable_event_callback(swReactor *reactor, swEvent *event)
 {
-    Socket *sock = (Socket *) event->socket->object;
-    sock->reactor->del(sock->reactor, event->fd);
-    sock->resume();
+    Socket *socket = (Socket *) event->socket->object;
+    socket->set_err(0);
+#ifdef SW_USE_OPENSSL
+    if (unlikely(socket->want_event != SW_EVENT_NULL))
+    {
+        if (socket->want_event == SW_EVENT_READ)
+        {
+            socket->write_co->resume();
+        }
+    }
+    else
+#endif
+    {
+        socket->read_co->resume();
+    }
     return SW_OK;
 }
 
+int Socket::writable_event_callback(swReactor *reactor, swEvent *event)
+{
+    Socket *socket = (Socket *) event->socket->object;
+    socket->set_err(0);
+#ifdef SW_USE_OPENSSL
+    if (unlikely(socket->want_event != SW_EVENT_NULL))
+    {
+        if (socket->want_event == SW_EVENT_WRITE)
+        {
+            socket->read_co->resume();
+        }
+    }
+    else
+#endif
+    {
+        socket->write_co->resume();
+    }
+    return SW_OK;
+}
+
+int Socket::error_event_callback(swReactor *reactor, swEvent *event)
+{
+    Socket *socket = (Socket *) event->socket->object;
+    if (socket->write_co)
+    {
+        socket->set_err(0);
+        socket->write_co->resume();
+    }
+    if (socket->read_co)
+    {
+        socket->set_err(0);
+        socket->read_co->resume();
+    }
+    return SW_OK;
+}
+
+bool Socket::add_event(const enum swEvent_type event)
+{
+    bool ret = true;
+    if (likely(!(socket->events & event)))
+    {
+        if (socket->removed)
+        {
+            ret = reactor->add(reactor, socket->fd, SW_FD_CORO_SOCKET | event) == SW_OK;
+        }
+        else
+        {
+            ret = reactor->set(reactor, socket->fd, SW_FD_CORO_SOCKET | socket->events | event) == SW_OK;
+        }
+    }
+    set_err(ret ? 0 : errno);
+    return ret;
+}
+
+bool Socket::wait_event(const enum swEvent_type event, const void **__buf, size_t __n)
+{
+    enum swEvent_type added_event = event;
+    Coroutine *co = Coroutine::get_current();
+    if (unlikely(!co))
+    {
+        swError("Socket::yield() must be called in the coroutine.");
+    }
+#ifdef SW_USE_OPENSSL
+    if (unlikely(socket->ssl && ((event == SW_EVENT_READ && socket->ssl_want_write) || (event == SW_EVENT_WRITE && socket->ssl_want_read))))
+    {
+        if (likely(socket->ssl_want_write && add_event(SW_EVENT_WRITE)))
+        {
+            want_event = SW_EVENT_WRITE;
+        }
+        else if (socket->ssl_want_read && add_event(SW_EVENT_READ))
+        {
+            want_event = SW_EVENT_READ;
+        }
+        else
+        {
+            return false;
+        }
+        added_event = want_event;
+    }
+    else
+#endif
+    if (unlikely(!add_event(event)))
+    {
+        return false;
+    }
+    swTraceLog(
+        SW_TRACE_SOCKET, "socket#%d blongs to cid#%ld is waiting for %s event",
+        socket->fd, co->get_cid(),
+#ifdef SW_USE_OPENSSL
+        socket->ssl_want_read ? "SSL READ" : socket->ssl_want_write ? "SSL WRITE" :
+#endif
+        event == SW_EVENT_READ ? "READ" : "WRITE"
+    );
+    if (likely(event == SW_EVENT_READ))
+    {
+        read_co = co;
+        read_co->yield();
+        read_co = nullptr;
+    }
+    else // if (event == SW_EVENT_WRITE)
+    {
+        if (unlikely(__n > 0 && *__buf != get_write_buffer()->str))
+        {
+            swString_clear(write_buffer);
+            swString_append_ptr(write_buffer, (const char *) *__buf, __n);
+            *__buf = write_buffer->str;
+        }
+        write_co = co;
+        write_co->yield();
+        write_co = nullptr;
+    }
+#ifdef SW_USE_OPENSSL
+    // maybe read_co and write_co are all waiting for the same event when we use SSL
+    if (likely(want_event == SW_EVENT_NULL || !has_bound()))
+#endif
+    {
+        if (likely(added_event == SW_EVENT_READ))
+        {
+            swReactor_remove_read_event(reactor, socket->fd);
+        }
+        else // if (added_event == SW_EVENT_WRITE)
+        {
+            swReactor_remove_write_event(reactor, socket->fd);
+        }
+    }
+#ifdef SW_USE_OPENSSL
+    want_event = SW_EVENT_NULL;
+#endif
+    swTraceLog(
+        SW_TRACE_SOCKET, "socket#%d blongs to cid#%ld trigger %s event",
+        socket->fd, co->get_cid(), socket->closed ? "CLOSE" :
+        errCode ? errCode == ETIMEDOUT ? "TIMEOUT" : "ERROR" :
+        added_event == SW_EVENT_READ ? "READ" : "WRITE"
+    );
+    return !socket->closed && !errCode;
+}
+
 bool Socket::socks5_handshake()
 {
     swSocks5 *ctx = socks5_proxy;
@@ -95,8 +254,7 @@ bool Socket::socks5_handshake()
         }
         if (status != 0)
         {
-            swoole_error_log(SW_LOG_NOTICE, SW_ERROR_SOCKS5_AUTH_FAILED,
-                    "SOCKS username/password authentication failed.");
+            swoole_error_log(SW_LOG_NOTICE, SW_ERROR_SOCKS5_AUTH_FAILED, "SOCKS username/password authentication failed.");
             return false;
         }
         goto send_connect_request;
@@ -184,13 +342,13 @@ bool Socket::http_proxy_handshake()
     {
         char auth_buf[256];
         char encode_buf[512];
-        n = snprintf(
+        n = sw_snprintf(
             auth_buf, sizeof(auth_buf), "%*s:%*s",
             http_proxy->l_user, http_proxy->user,
             http_proxy->l_password, http_proxy->password
         );
         swBase64_encode((unsigned char *) auth_buf, n, encode_buf);
-        n = snprintf(
+        n = sw_snprintf(
             http_proxy->buf, sizeof(http_proxy->buf),
             "CONNECT %*s:%d HTTP/1.1\r\nProxy-Authorization:Basic %s\r\n\r\n",
             http_proxy->l_target_host, http_proxy->target_host, http_proxy->target_port, encode_buf
@@ -198,7 +356,7 @@ bool Socket::http_proxy_handshake()
     }
     else
     {
-        n = snprintf(
+        n = sw_snprintf(
             http_proxy->buf, sizeof(http_proxy->buf),
             "CONNECT %*s:%d HTTP/1.1\r\n\r\n",
             http_proxy->l_target_host, http_proxy->target_host, http_proxy->target_port
@@ -274,22 +432,37 @@ bool Socket::http_proxy_handshake()
     return false;
 }
 
-static inline int socket_connect(int fd, const struct sockaddr *addr, socklen_t len)
+void Socket::init_sock_type(enum swSocket_type _type)
 {
-    int retval;
-    while (1)
+    type = _type;
+    switch (type)
     {
-        retval = ::connect(fd, addr, len);
-        if (retval < 0)
-        {
-            if (errno == EINTR)
-            {
-                continue;
-            }
-        }
+    case SW_SOCK_TCP6:
+        sock_domain = AF_INET6;
+        sock_type = SOCK_STREAM;
+        break;
+    case SW_SOCK_UNIX_STREAM:
+        sock_domain = AF_UNIX;
+        sock_type = SOCK_STREAM;
+        break;
+    case SW_SOCK_UDP:
+        sock_domain = AF_INET;
+        sock_type = SOCK_DGRAM;
+        break;
+    case SW_SOCK_UDP6:
+        sock_domain = AF_INET6;
+        sock_type = SOCK_DGRAM;
+        break;
+    case SW_SOCK_UNIX_DGRAM:
+        sock_domain = AF_UNIX;
+        sock_type = SOCK_DGRAM;
+        break;
+    case SW_SOCK_TCP:
+    default:
+        sock_domain = AF_INET;
+        sock_type = SOCK_STREAM;
         break;
     }
-    return retval;
 }
 
 void Socket::init_sock()
@@ -329,9 +502,9 @@ void Socket::init_sock(int _fd)
     swSetNonBlock(socket->fd);
     if (!swReactor_handle_isset(reactor, SW_FD_CORO_SOCKET))
     {
-        reactor->setHandle(reactor, SW_FD_CORO_SOCKET | SW_EVENT_READ, event_callback);
-        reactor->setHandle(reactor, SW_FD_CORO_SOCKET | SW_EVENT_WRITE, event_callback);
-        reactor->setHandle(reactor, SW_FD_CORO_SOCKET | SW_EVENT_ERROR, event_callback);
+        reactor->setHandle(reactor, SW_FD_CORO_SOCKET | SW_EVENT_READ, readable_event_callback);
+        reactor->setHandle(reactor, SW_FD_CORO_SOCKET | SW_EVENT_WRITE, writable_event_callback);
+        reactor->setHandle(reactor, SW_FD_CORO_SOCKET | SW_EVENT_ERROR, error_event_callback);
     }
 }
 
@@ -376,67 +549,53 @@ Socket::Socket(int _fd, Socket *server_sock)
     socket->fdtype = SW_FD_CORO_SOCKET;
 }
 
-void Socket::set_timer(timer_levels _timer_level, double _timeout)
-{
-    if (_timeout == 0)
-    {
-        _timeout = timeout;
-    }
-    if (!timer && _timeout > 0)
-    {
-        timer_level = _timer_level;
-        timer = swTimer_add(&SwooleG.timer, (long) (_timeout * 1000), 0, this, timer_callback);
-    }
-}
-
-void Socket::del_timer(timer_levels _timer_level)
-{
-    if (timer && _timer_level == timer_level)
-    {
-        swTimer_del(&SwooleG.timer, timer);
-        timer = nullptr;
-    }
-}
-
 bool Socket::connect(const struct sockaddr *addr, socklen_t addrlen)
 {
-    if (unlikely(!is_available()))
+    if (unlikely(!is_available(SW_EVENT_RDWR)))
     {
         return false;
     }
-    int retval = socket_connect(socket->fd, addr, addrlen);
-    if (retval == -1)
+    int retval;
+    do {
+        retval = ::connect(socket->fd, addr, addrlen);
+    } while (retval < 0 && errno == EINTR);
+    if (retval < 0)
     {
         if (errno != EINPROGRESS)
         {
             set_err(errno);
             return false;
         }
-        if (!wait_writeable())
+        else
         {
-            return false;
-        }
-        //Connection is closed
-        if (socket->closed)
-        {
-            set_err(ECONNABORTED);
-            return false;
-        }
-        socklen_t len = sizeof(errCode);
-        if (getsockopt(socket->fd, SOL_SOCKET, SO_ERROR, &errCode, &len) < 0 || errCode != 0)
-        {
-            set_err(errCode);
-            return false;
+            timer timer(&write_timer, timeout, this, timer_callback);
+            if (!timer() || !wait_event(SW_EVENT_WRITE))
+            {
+                if (socket->closed)
+                {
+                    set_err(ECONNABORTED);
+                }
+                return false;
+            }
+            else
+            {
+                socklen_t len = sizeof(errCode);
+                if (getsockopt(socket->fd, SOL_SOCKET, SO_ERROR, &errCode, &len) < 0 || errCode != 0)
+                {
+                    set_err(errCode);
+                    return false;
+                }
+            }
         }
     }
-    set_err(0);
     socket->active = 1;
+    set_err(0);
     return true;
 }
 
 bool Socket::connect(string _host, int _port, int flags)
 {
-    if (unlikely(!is_available()))
+    if (unlikely(!is_available(SW_EVENT_RDWR)))
     {
         return false;
     }
@@ -602,72 +761,79 @@ ssize_t Socket::peek(void *__buf, size_t __n)
 
 ssize_t Socket::recv(void *__buf, size_t __n)
 {
-    if (unlikely(!is_available()))
+    if (unlikely(!is_available(SW_EVENT_READ)))
     {
         return -1;
     }
-    ssize_t retval = swConnection_recv(socket, __buf, __n, 0);
-    while (retval < 0 && swConnection_error(errno) == SW_WAIT)
-    {
-        if (!wait_readable())
-        {
-            return -1;
-        }
+    ssize_t retval;
+    timer timer(&read_timer, timeout, this, timer_callback);
+    do {
         retval = swConnection_recv(socket, __buf, __n, 0);
+    } while (retval < 0 && swConnection_error(errno) == SW_WAIT && timer() && wait_event(SW_EVENT_READ));
+    set_err(retval < 0 ? errno : 0);
+    return retval;
+}
+
+ssize_t Socket::send(const void *__buf, size_t __n)
+{
+    if (unlikely(!is_available(SW_EVENT_WRITE)))
+    {
+        return -1;
     }
+    ssize_t retval;
+    timer timer(&write_timer, timeout, this, timer_callback);
+    do {
+        retval = swConnection_send(socket, (void *) __buf, __n, 0);
+    } while (retval < 0 && swConnection_error(errno) == SW_WAIT && timer() && wait_event(SW_EVENT_WRITE));
     set_err(retval < 0 ? errno : 0);
     return retval;
 }
 
 ssize_t Socket::read(void *__buf, size_t __n)
 {
-    if (unlikely(!is_available()))
+    if (unlikely(!is_available(SW_EVENT_READ)))
     {
         return -1;
     }
-    ssize_t retval = ::read(socket->fd, __buf, __n);
-    while (retval < 0 && swConnection_error(errno) == SW_WAIT)
-    {
-        if (!wait_readable())
-        {
-            return -1;
-        }
+    ssize_t retval;
+    timer timer(&read_timer, timeout, this, timer_callback);
+    do {
         retval = ::read(socket->fd, __buf, __n);
-    }
+    } while (retval < 0 && swConnection_error(errno) == SW_WAIT && timer() && wait_event(SW_EVENT_READ));
     set_err(retval < 0 ? errno : 0);
     return retval;
 }
 
 ssize_t Socket::write(const void *__buf, size_t __n)
 {
-    if (unlikely(!is_available()))
+    if (unlikely(!is_available(SW_EVENT_WRITE)))
     {
         return -1;
     }
-    ssize_t retval = ::write(socket->fd, (void *) __buf, __n);
-    while (retval < 0 && swConnection_error(errno) == SW_WAIT)
-    {
-        if (!wait_writeable(&__buf, __n))
-        {
-            return -1;
-        }
+    ssize_t retval;
+    timer timer(&write_timer, timeout, this, timer_callback);
+    do {
         retval = ::write(socket->fd, (void *) __buf, __n);
-    }
+    } while (retval < 0 && swConnection_error(errno) == SW_WAIT && timer() && wait_event(SW_EVENT_WRITE));
     set_err(retval < 0 ? errno : 0);
     return retval;
 }
 
 ssize_t Socket::recv_all(void *__buf, size_t __n)
 {
+    if (unlikely(!is_available(SW_EVENT_READ)))
+    {
+        return -1;
+    }
     ssize_t retval, total_bytes = 0;
-    if (unlikely(!is_available()))
+    timer timer(&read_timer, timeout, this, timer_callback);
+    if (unlikely(!timer()))
     {
         return -1;
     }
-    set_timer(TIMER_LV_MULTI);
     while (true)
     {
-        retval = recv((char*) __buf + total_bytes, __n - total_bytes);
+        retval = recv((char *) __buf + total_bytes, __n - total_bytes);
         if (retval <= 0)
         {
             if (total_bytes == 0)
@@ -682,21 +848,24 @@ ssize_t Socket::recv_all(void *__buf, size_t __n)
             break;
         }
     }
-    del_timer(TIMER_LV_MULTI);
     return total_bytes;
 }
 
 ssize_t Socket::send_all(const void *__buf, size_t __n)
 {
+    if (unlikely(!is_available(SW_EVENT_WRITE)))
+    {
+        return -1;
+    }
     ssize_t retval, total_bytes = 0;
-    if (unlikely(!is_available()))
+    timer timer(&write_timer, timeout, this, timer_callback);
+    if (unlikely(!timer()))
     {
         return -1;
     }
-    set_timer(TIMER_LV_MULTI);
     while (true)
     {
-        retval = send((char*) __buf + total_bytes, __n - total_bytes);
+        retval = send((char *) __buf + total_bytes, __n - total_bytes);
         if (retval <= 0)
         {
             if (total_bytes == 0)
@@ -711,73 +880,45 @@ ssize_t Socket::send_all(const void *__buf, size_t __n)
             break;
         }
     }
-    del_timer(TIMER_LV_MULTI);
     return total_bytes;
 }
 
-ssize_t Socket::send(const void *__buf, size_t __n)
-{
-    if (unlikely(!is_available()))
-    {
-        return -1;
-    }
-    ssize_t retval = swConnection_send(socket, (void *) __buf, __n, 0);
-    while (retval < 0 && swConnection_error(errno) == SW_WAIT)
-    {
-        if (!wait_writeable(&__buf, __n))
-        {
-            return -1;
-        }
-        retval = swConnection_send(socket, (void *) __buf, __n, 0);
-    }
-    set_err(retval < 0 ? errno : 0);
-    return retval;
-}
-
 /**
  * Notice: you must use non-global buffer here (or else it may be changed after yield)
  */
-ssize_t Socket::sendmsg(const struct msghdr *msg, int flags)
+ssize_t Socket::recvmsg(struct msghdr *msg, int flags)
 {
-    if (unlikely(!is_available()))
+    if (unlikely(!is_available(SW_EVENT_READ)))
     {
         return -1;
     }
-    ssize_t retval = ::sendmsg(socket->fd, msg, flags);
-    while (retval < 0 && swConnection_error(errno) == SW_WAIT)
-    {
-        if (!wait_writeable())
-        {
-            return -1;
-        }
-        retval = ::sendmsg(socket->fd, msg, flags);
-    }
+    ssize_t retval;
+    timer timer(&read_timer, timeout, this, timer_callback);
+    do {
+        retval = ::recvmsg(socket->fd, msg, flags);
+    } while (retval < 0 && swConnection_error(errno) == SW_WAIT && timer() && wait_event(SW_EVENT_READ));
     set_err(retval < 0 ? errno : 0);
     return retval;
 }
 
-ssize_t Socket::recvmsg(struct msghdr *msg, int flags)
+ssize_t Socket::sendmsg(const struct msghdr *msg, int flags)
 {
-    if (unlikely(!is_available()))
+    if (unlikely(!is_available(SW_EVENT_WRITE)))
     {
         return -1;
     }
-    ssize_t retval = ::recvmsg(socket->fd, msg, flags);
-    while (retval < 0 && swConnection_error(errno) == SW_WAIT)
-    {
-        if (!wait_readable())
-        {
-            return -1;
-        }
-        retval = ::recvmsg(socket->fd, msg, flags);
-    }
+    ssize_t retval;
+    timer timer(&write_timer, timeout, this, timer_callback);
+    do {
+        retval = ::sendmsg(socket->fd, msg, flags);
+    } while (retval < 0 && swConnection_error(errno) == SW_WAIT && timer() && wait_event(SW_EVENT_WRITE));
     set_err(retval < 0 ? errno : 0);
     return retval;
 }
 
 bool Socket::bind(std::string address, int port)
 {
-    if (unlikely(!is_available()))
+    if (unlikely(!is_available(SW_EVENT_NULL)))
     {
         return false;
     }
@@ -864,7 +1005,7 @@ bool Socket::bind(std::string address, int port)
 
 bool Socket::listen(int backlog)
 {
-    if (unlikely(!is_available()))
+    if (unlikely(!is_available(SW_EVENT_NULL)))
     {
         return false;
     }
@@ -890,7 +1031,7 @@ bool Socket::listen(int backlog)
 
 Socket* Socket::accept()
 {
-    if (unlikely(!is_available()))
+    if (unlikely(!is_available(SW_EVENT_READ)))
     {
         return nullptr;
     }
@@ -898,7 +1039,8 @@ Socket* Socket::accept()
     int conn = swSocket_accept(socket->fd, &client_addr);
     if (conn < 0 && errno == EAGAIN)
     {
-        if (!wait_readable())
+        timer timer(&read_timer, timeout, this, timer_callback);
+        if (!timer() || !wait_event(SW_EVENT_READ))
         {
             return nullptr;
         }
@@ -921,74 +1063,20 @@ Socket* Socket::accept()
 #ifdef SW_USE_OPENSSL
     if (open_ssl)
     {
-        if (swSSL_create(client_sock->socket, ssl_context, 0) < 0)
+        if (swSSL_create(client_sock->socket, ssl_context, 0) < 0 || !client_sock->ssl_accept())
         {
-            _delete: delete client_sock;
+            client_sock->close();
             return nullptr;
         }
-        if (client_sock->ssl_accept() == false)
-        {
-            goto _delete;
-        }
     }
 #endif
     return client_sock;
 }
 
-bool Socket::shutdown(int __how)
-{
-    if (__how == SHUT_RD && !shutdown_read)
-    {
-        if (::shutdown(socket->fd, SHUT_RD) == 0)
-        {
-            shutdown_read = true;
-            return true;
-        }
-    }
-    else if (__how == SHUT_WR && !shutdown_write)
-    {
-        if (::shutdown(socket->fd, SHUT_WR) == 0)
-        {
-            shutdown_write = true;
-            return true;
-        }
-    }
-    else if (__how == SHUT_RDWR && !shutdown_read && !shutdown_write)
-    {
-        if (::shutdown(socket->fd, SHUT_RDWR) == 0)
-        {
-            shutdown_read = shutdown_write = true;
-            return true;
-        }
-    }
-    set_err(errno);
-    return false;
-}
-
-bool Socket::close()
-{
-    // TODO: waiting on review
-    if (!socket->closed)
-    {
-        socket->closed = 1;
-    }
-    if (socket->active)
-    {
-        shutdown();
-        socket->active = 0;
-    }
-    if (coroutine)
-    {
-        reactor->del(reactor, socket->fd);
-        resume();
-    }
-    return true;
-}
-
 #ifdef SW_USE_OPENSSL
 bool Socket::ssl_handshake()
 {
-    if (unlikely(!is_available()))
+    if (unlikely(!is_available(SW_EVENT_RDWR)))
     {
         return -1;
     }
@@ -1034,15 +1122,15 @@ bool Socket::ssl_handshake()
 
     while (true)
     {
-        int retval = swSSL_connect(socket);
-        if (retval < 0)
+        if (swSSL_connect(socket) < 0)
         {
-            set_err(SwooleG.error);
+            set_err(errno);
             return false;
         }
         if (socket->ssl_state == SW_SSL_STATE_WAIT_STREAM)
         {
-            if (!wait_readable())
+            timer timer(&read_timer, timeout, this, timer_callback);
+            if (!timer() || !wait_event(SW_EVENT_READ))
             {
                 return false;
             }
@@ -1065,31 +1153,17 @@ bool Socket::ssl_handshake()
 
 bool Socket::ssl_accept()
 {
+    int retval;
+    timer timer(&read_timer, timeout, this, timer_callback);
     open_ssl = true;
-    while (true)
-    {
-        int retval = swSSL_accept(socket);
-        if (retval == SW_ERROR)
-        {
-            return false;
-        }
-        else if (retval == SW_READY)
-        {
-            return true;
-        }
-        if (!wait_readable())
-        {
-            return -1;
-        }
-    }
+    do {
+        retval = swSSL_accept(socket);
+    } while (retval == SW_WAIT && timer() && wait_event(SW_EVENT_READ));
+    return retval == SW_READY;
 }
 
 int Socket::ssl_verify(bool allow_self_signed)
 {
-    if (unlikely(!is_available()))
-    {
-        return -1;
-    }
     if (swSSL_verify(socket, allow_self_signed) < 0)
     {
         return SW_ERR;
@@ -1104,7 +1178,7 @@ int Socket::ssl_verify(bool allow_self_signed)
 
 bool Socket::sendfile(char *filename, off_t offset, size_t length)
 {
-    if (unlikely(!is_available()))
+    if (unlikely(!is_available(SW_EVENT_WRITE)))
     {
         return false;
     }
@@ -1132,8 +1206,8 @@ bool Socket::sendfile(char *filename, off_t offset, size_t length)
         length = offset + length;
     }
 
+    timer timer(&read_timer, timeout, this, timer_callback);
     int n, sendn;
-    set_timer(TIMER_LV_MULTI);
     while ((size_t) offset < length)
     {
         sendn = (length - offset > SW_SENDFILE_CHUNK_SIZE) ? SW_SENDFILE_CHUNK_SIZE : length - offset;
@@ -1164,20 +1238,19 @@ bool Socket::sendfile(char *filename, off_t offset, size_t length)
             ::close(file_fd);
             return false;
         }
-        if (!wait_writeable())
+        if (!timer() || !wait_event(SW_EVENT_WRITE))
         {
             ::close(file_fd);
             return false;
         }
     }
-    del_timer(TIMER_LV_MULTI);
     ::close(file_fd);
     return true;
 }
 
 ssize_t Socket::sendto(char *address, int port, char *data, int len)
 {
-    if (unlikely(!is_available()))
+    if (unlikely(!is_available(SW_EVENT_WRITE)))
     {
         return -1;
     }
@@ -1198,7 +1271,7 @@ ssize_t Socket::sendto(char *address, int port, char *data, int len)
 
 ssize_t Socket::recvfrom(void *__buf, size_t __n)
 {
-    if (unlikely(!is_available()))
+    if (unlikely(!is_available(SW_EVENT_READ)))
     {
         return -1;
     }
@@ -1208,23 +1281,15 @@ ssize_t Socket::recvfrom(void *__buf, size_t __n)
 
 ssize_t Socket::recvfrom(void *__buf, size_t __n, struct sockaddr* _addr, socklen_t *_socklen)
 {
-    if (unlikely(!is_available()))
+    if (unlikely(!is_available(SW_EVENT_READ)))
     {
         return -1;
     }
-    ssize_t retval = ::recvfrom(socket->fd, __buf, __n, 0, _addr, _socklen);
-    while (retval < 0 && errno == EINTR)
-    {
+    ssize_t retval;
+    timer timer(&read_timer, timeout, this, timer_callback);
+    do {
         retval = ::recvfrom(socket->fd, __buf, __n, 0, _addr, _socklen);
-    }
-    while (retval < 0 && swConnection_error(errno) == SW_WAIT)
-    {
-        if (!wait_readable())
-        {
-            return -1;
-        }
-        retval = ::recvfrom(socket->fd, __buf, __n, 0, _addr, _socklen);
-    }
+    } while (retval < 0 && ((errno == EINTR) || (swConnection_error(errno) == SW_WAIT && timer() && wait_event(SW_EVENT_READ))));
     set_err(retval < 0 ? errno : 0);
     return retval;
 }
@@ -1232,16 +1297,22 @@ ssize_t Socket::recvfrom(void *__buf, size_t __n, struct sockaddr* _addr, sockle
 /**
  * recv packet with protocol
  */
-ssize_t Socket::recv_packet()
+ssize_t Socket::recv_packet(double timeout)
 {
-    if (unlikely(!is_available()))
+    if (unlikely(!is_available(SW_EVENT_READ)))
     {
         return -1;
     }
 
-    get_read_buffer();
     ssize_t buf_len = SW_BUFFER_SIZE_STD;
     ssize_t retval;
+    timer timer(&read_timer, timeout == 0 ? this->timeout : timeout, this, timer_callback);
+
+    if (unlikely(!timer()))
+    {
+        return -1;
+    }
+    get_read_buffer();
 
     if (open_length_check)
     {
@@ -1254,13 +1325,13 @@ ssize_t Socket::recv_packet()
         uint32_t header_len = protocol.package_length_offset + protocol.package_length_size;
         if (read_buffer->length > 0)
         {
-            if (read_buffer->length < header_len)
+            if (read_buffer->length >= header_len || protocol.package_length_type == '\0')
             {
-                goto _recv_header;
+                goto _get_length;
             }
             else
             {
-                goto _get_length;
+                goto _recv_header;
             }
         }
 
@@ -1268,7 +1339,7 @@ ssize_t Socket::recv_packet()
         retval = recv(read_buffer->str + read_buffer->length, header_len - read_buffer->length);
         if (retval <= 0)
         {
-            return 0;
+            return retval;
         }
         else
         {
@@ -1307,7 +1378,7 @@ ssize_t Socket::recv_packet()
         }
         else if ((size_t) buf_len < read_buffer->length)
         {
-            //unprocessed data
+            //unprocessed data (offset will always be zero)
             read_buffer->length -= buf_len;
             read_buffer->offset = buf_len;
             return buf_len;
@@ -1427,14 +1498,103 @@ ssize_t Socket::recv_packet()
     return retval;
 }
 
+// TODO: resume read_co/write_co
+bool Socket::shutdown(int __how)
+{
+    set_err(0);
+    if (!is_connect() || (__how == SHUT_RD && shutdown_read) || (__how == SHUT_WR && shutdown_write))
+    {
+        errno = ENOTCONN;
+    }
+    else
+    {
+#ifdef SW_USE_OPENSSL
+        if (socket->ssl)
+        {
+            SSL_set_quiet_shutdown(socket->ssl, 1);
+            SSL_shutdown(socket->ssl);
+        }
+#endif
+        if (::shutdown(socket->fd, __how) == 0 || errno == ENOTCONN)
+        {
+            if (errno == ENOTCONN)
+            {
+                // connection reset by server side
+                __how = SHUT_RDWR;
+            }
+            switch (__how)
+            {
+            case SHUT_RD:
+                shutdown_read = true;
+                break;
+            case SHUT_WR:
+                shutdown_write = true;
+                break;
+            default:
+                shutdown_read = shutdown_write = true;
+            }
+            if (shutdown_read && shutdown_write)
+            {
+                socket->active = 0;
+            }
+            return true;
+        }
+    }
+    set_err(errno);
+    return false;
+}
+
+bool Socket::close()
+{
+    bool ret = true;
+    if (has_bound())
+    {
+        if (socket->closed)
+        {
+            // close operation is in processing
+            return false;
+        }
+        if (socket->active)
+        {
+            ret = shutdown();
+        }
+        if (!socket->closed)
+        {
+            socket->closed = 1;
+        }
+        if (write_co)
+        {
+            set_err(ECONNRESET);
+            write_co->resume();
+        }
+        if (read_co)
+        {
+            set_err(ECONNRESET);
+            read_co->resume();
+        }
+    }
+    else
+    {
+        int fd = socket->fd;
+        socket->fd = -1;
+        delete this;
+        ret = ::close(fd) == 0;
+    }
+    return ret;
+}
+
+/**
+ * Notice:
+ * the destructor should only be called when the construct fails
+ * If you want to safe release the socket, you should call Soskcet::close
+ */
 Socket::~Socket()
 {
-    int fd;
     if (socket == nullptr)
     {
-        // construct failed
         return;
     }
+    SW_ASSERT(!has_bound());
     if (read_buffer)
     {
         swString_free(read_buffer);
@@ -1443,14 +1603,7 @@ Socket::~Socket()
     {
         swString_free(write_buffer);
     }
-    if (sock_domain == AF_UNIX && !bind_address.empty())
-    {
-        unlink(bind_address_info.addr.un.sun_path);
-    }
-    if (sock_type == SW_SOCK_UNIX_DGRAM)
-    {
-        unlink(socket->info.addr.un.sun_path);
-    }
+    /* {{{ release socket resources */
 #ifdef SW_USE_OPENSSL
     if (socket->ssl)
     {
@@ -1459,6 +1612,7 @@ Socket::~Socket()
     if (ssl_context)
     {
         swSSL_free_context(ssl_context);
+        ssl_context = nullptr;
         if (ssl_option.cert_file)
         {
             sw_free(ssl_option.cert_file);
@@ -1485,6 +1639,7 @@ Socket::~Socket()
         {
             sw_free(ssl_option.capath);
         }
+        ssl_option = {0};
     }
 #endif
     if (socket->in_buffer)
@@ -1495,13 +1650,26 @@ Socket::~Socket()
     {
         swBuffer_free(socket->out_buffer);
     }
-    fd = socket->fd;
-    if (socket->removed == 0)
+    if (sock_domain == AF_UNIX && !bind_address.empty())
     {
-        reactor->del(reactor, fd);
+        unlink(bind_address_info.addr.un.sun_path);
+        bind_address_info = {{}, 0};
+    }
+    if (sock_type == SW_SOCK_UNIX_DGRAM)
+    {
+        unlink(socket->info.addr.un.sun_path);
+    }
+    if (unlikely(socket->fd > 0))
+    {
+        if (!socket->removed)
+        {
+            reactor->del(reactor, socket->fd);
+        }
+        ::close(socket->fd);
     }
     bzero(socket, sizeof(swConnection));
+    socket->fd = -1;
     socket->removed = 1;
     socket->closed = 1;
-    ::close(fd);
+    /* }}} */
 }
