@@ -56,7 +56,11 @@ static sw_inline int swProtocol_split_package_by_eof(swProtocol *protocol, swCon
 #endif
 
     int eof_pos;
-    if (buffer->length - buffer->offset < protocol->package_eof_len)
+    _find_eof: if (buffer->length < protocol->package_eof_len)
+    {
+        return SW_CONTINUE;
+    }
+    else if (buffer->length - buffer->offset < protocol->package_eof_len)
     {
         eof_pos = -1;
     }
@@ -92,45 +96,24 @@ static sw_inline int swProtocol_split_package_by_eof(swProtocol *protocol, swCon
         char *remaining_data = buffer->str + length;
         swTraceLog(SW_TRACE_EOF_PROTOCOL, "#[5] count=%d, remaining_length=%d", count, remaining_length);
 
-        while (1)
+        if (remaining_length > 0)
+        {
+            memmove(buffer->str, remaining_data, remaining_length);
+            buffer->length = remaining_length;
+            buffer->offset = 0;
+            goto _find_eof;
+        }
+        else
         {
-            if (remaining_length < protocol->package_eof_len)
-            {
-                goto wait_more_data;
-            }
-            eof_pos = swoole_strnpos(remaining_data, remaining_length, protocol->package_eof, protocol->package_eof_len);
-            if (eof_pos < 0)
-            {
-                wait_more_data:
-                swTraceLog(SW_TRACE_EOF_PROTOCOL, "#[1] count=%d, remaining_length=%d, length=%d", count, remaining_length, length);
-                memmove(buffer->str, remaining_data, remaining_length);
-                buffer->length = remaining_length;
-                buffer->offset = 0;
 #ifdef SW_USE_OPENSSL
-                if (conn->ssl)
-                {
-                    return SW_CONTINUE;
-                }
-                else
-#endif
-                {
-                    return SW_OK;
-                }
+            if (conn->ssl)
+            {
+                return SW_CONTINUE;
             }
             else
+#endif
             {
-                length = eof_pos + protocol->package_eof_len;
-                if (protocol->onPackage(conn, remaining_data, length) < 0)
-                {
-                    return SW_CLOSE;
-                }
-                if (conn->removed)
-                {
-                    return SW_OK;
-                }
-                swTraceLog(SW_TRACE_EOF_PROTOCOL, "#[2] count=%d, remaining_length=%d, length=%d", count, remaining_length, length);
-                remaining_data += length;
-                remaining_length -= length;
+                return SW_OK;
             }
         }
     }
