@@ -159,7 +159,7 @@ static void swReactor_onTimeout_and_Finish(swReactor *reactor)
         }
     }
     //not server, the event loop is empty
-    if (SwooleG.serv == NULL && swReactor_empty(reactor))
+    if ((SwooleG.serv == NULL || swIsUserWorker()) && swReactor_empty(reactor))
     {
         reactor->running = 0;
     }
