@@ -60,12 +60,16 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_create, 0, 0, 1)
     ZEND_ARG_VARIADIC_INFO(0, params)
 ZEND_END_ARG_INFO()
 
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_exec, 0, 0, 1)
-    ZEND_ARG_INFO(0, command)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_resume, 0, 0, 1)
+    ZEND_ARG_INFO(0, cid)
 ZEND_END_ARG_INFO()
 
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_resume, 0, 0, 1)
-    ZEND_ARG_INFO(0, uid)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_exists, 0, 0, 1)
+    ZEND_ARG_INFO(0, cid)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_exec, 0, 0, 1)
+    ZEND_ARG_INFO(0, command)
 ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_sleep, 0, 0, 1)
@@ -119,18 +123,20 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_statvfs, 0, 0, 1)
     ZEND_ARG_INFO(0, path)
 ZEND_END_ARG_INFO()
 
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_getBackTrace, 0, 0, 1)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_getBackTrace, 0, 0, 0)
     ZEND_ARG_INFO(0, cid)
     ZEND_ARG_INFO(0, options)
     ZEND_ARG_INFO(0, limit)
 ZEND_END_ARG_INFO()
 
 static PHP_METHOD(swoole_coroutine_util, set);
+static PHP_METHOD(swoole_coroutine_util, exists);
 static PHP_METHOD(swoole_coroutine_util, yield);
 static PHP_METHOD(swoole_coroutine_util, resume);
 static PHP_METHOD(swoole_coroutine_util, stats);
 static PHP_METHOD(swoole_coroutine_util, getCid);
-static PHP_METHOD(swoole_coroutine_util, listCoroutines);
+static PHP_METHOD(swoole_coroutine_util, getPcid);
+static PHP_METHOD(swoole_coroutine_util, list);
 static PHP_METHOD(swoole_coroutine_util, sleep);
 static PHP_METHOD(swoole_coroutine_util, fread);
 static PHP_METHOD(swoole_coroutine_util, fgets);
@@ -177,12 +183,14 @@ static const zend_function_entry swoole_coroutine_util_methods[] =
     ZEND_FENTRY(gethostbyname, ZEND_FN(swoole_coroutine_gethostbyname), arginfo_swoole_coroutine_gethostbyname, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     ZEND_FENTRY(defer, ZEND_FN(swoole_coroutine_defer), arginfo_swoole_coroutine_defer, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_coroutine_util, set, arginfo_swoole_coroutine_set, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+    PHP_ME(swoole_coroutine_util, exists, arginfo_swoole_coroutine_exists, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_coroutine_util, yield, arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_MALIAS(swoole_coroutine_util, suspend, yield, arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_coroutine_util, resume, arginfo_swoole_coroutine_resume, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_coroutine_util, stats, arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_coroutine_util, getCid, arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_MALIAS(swoole_coroutine_util, getuid, getCid, arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+    PHP_ME(swoole_coroutine_util, getPcid, arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_coroutine_util, sleep, arginfo_swoole_coroutine_sleep, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_coroutine_util, fread, arginfo_swoole_coroutine_fread, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_coroutine_util, fgets, arginfo_swoole_coroutine_fgets, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
@@ -192,7 +200,8 @@ static const zend_function_entry swoole_coroutine_util_methods[] =
     PHP_ME(swoole_coroutine_util, getaddrinfo, arginfo_swoole_coroutine_getaddrinfo, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_coroutine_util, statvfs, arginfo_swoole_coroutine_statvfs, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_coroutine_util, getBackTrace, arginfo_swoole_coroutine_getBackTrace, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-    PHP_ME(swoole_coroutine_util, listCoroutines, arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+    PHP_ME(swoole_coroutine_util, list, arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+    PHP_MALIAS(swoole_coroutine_util, listCoroutines, list, arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_FE_END
 };
 
@@ -332,9 +341,17 @@ static PHP_METHOD(swoole_exit_exception, getStatus)
     RETURN_ZVAL(sw_zend_read_property(Z_OBJCE_P(getThis()), getThis(), ZEND_STRL("status"), 1), 1, 0);
 }
 
-/*
- * suspend current coroutine
- */
+static PHP_METHOD(swoole_coroutine_util, exists)
+{
+    zend_long cid;
+
+    ZEND_PARSE_PARAMETERS_START(1, 1)
+        Z_PARAM_LONG(cid)
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+
+    RETURN_BOOL(Coroutine::get_by_cid(cid) != nullptr);
+}
+
 static PHP_METHOD(swoole_coroutine_util, yield)
 {
     Coroutine* co = Coroutine::get_current();
@@ -362,55 +379,46 @@ static PHP_METHOD(swoole_coroutine_util, set)
     vht = Z_ARRVAL_P(zset);
     if (php_swoole_array_get_value(vht, "max_coroutine", v))
     {
-        zend_long max_num;
-        convert_to_long(v);
-        max_num = Z_LVAL_P(v);
+        zend_long max_num = zval_get_long(v);
         PHPCoroutine::set_max_num(max_num <= 0 ? SW_DEFAULT_MAX_CORO_NUM : max_num);
     }
     if (php_swoole_array_get_value(vht, "c_stack_size", v) || php_swoole_array_get_value(vht, "stack_size", v))
     {
-        zend_long c_stack_size;
-        convert_to_long(v);
-        c_stack_size = Z_LVAL_P(v);
+        zend_long c_stack_size = zval_get_long(v);
         Coroutine::set_stack_size(c_stack_size);
     }
     if (php_swoole_array_get_value(vht, "socket_connect_timeout", v))
     {
-        convert_to_double(v);
-        if (Z_DVAL_P(v))
+        double _value = zval_get_double(v);
+        if (_value)
         {
-            PHPCoroutine::socket_connect_timeout = (double) Z_DVAL_P(v);
+            PHPCoroutine::socket_connect_timeout = _value;
         }
     }
     if (php_swoole_array_get_value(vht, "socket_timeout", v))
     {
-        convert_to_double(v);
-        if (Z_DVAL_P(v))
+        double _value = zval_get_double(v);
+        if (_value)
         {
-            PHPCoroutine::socket_timeout = (double) Z_DVAL_P(v);
+            PHPCoroutine::socket_timeout = _value;
         }
     }
     if (php_swoole_array_get_value(vht, "log_level", v))
     {
-        zend_long level;
-        convert_to_long(v);
-        level = Z_LVAL_P(v);
+        zend_long level = zval_get_long(v);
         SwooleG.log_level = (uint32_t) (level < 0 ? UINT32_MAX : level);
     }
     if (php_swoole_array_get_value(vht, "trace_flags", v))
     {
-        convert_to_long(v);
-        SwooleG.trace_flags = (uint32_t) MAX(0, Z_LVAL_P(v));
+        SwooleG.trace_flags = (uint32_t) MAX(0, zval_get_long(v));
     }
     if (php_swoole_array_get_value(vht, "dns_cache_expire", v))
     {
-        convert_to_long(v);
-        set_dns_cache_expire((time_t) Z_LVAL_P(v));
+        set_dns_cache_expire((time_t) zval_get_long(v));
     }
     if (php_swoole_array_get_value(vht, "dns_cache_capacity", v))
     {
-        convert_to_long(v);
-        set_dns_cache_capacity((size_t) Z_LVAL_P(v));
+        set_dns_cache_capacity((size_t) zval_get_long(v));
     }
     zval_ptr_dtor(zset);
 }
@@ -475,9 +483,16 @@ static PHP_METHOD(swoole_coroutine_util, resume)
 static PHP_METHOD(swoole_coroutine_util, stats)
 {
     array_init(return_value);
+    if (SwooleG.main_reactor)
+    {
+        add_assoc_long_ex(return_value, ZEND_STRL("event_num"), SwooleG.main_reactor->event_num);
+        add_assoc_long_ex(return_value, ZEND_STRL("signal_listener_num"), SwooleG.main_reactor->signal_listener_num);
+    }
+    add_assoc_long_ex(return_value, ZEND_STRL("aio_task_num"), SwooleAIO.task_num);
     add_assoc_long_ex(return_value, ZEND_STRL("c_stack_size"), Coroutine::get_stack_size());
     add_assoc_long_ex(return_value, ZEND_STRL("coroutine_num"), Coroutine::count());
     add_assoc_long_ex(return_value, ZEND_STRL("coroutine_peak_num"), Coroutine::get_peak_num());
+    add_assoc_long_ex(return_value, ZEND_STRL("coroutine_last_cid"), Coroutine::get_last_cid());
 }
 
 static PHP_METHOD(swoole_coroutine_util, getCid)
@@ -485,6 +500,11 @@ static PHP_METHOD(swoole_coroutine_util, getCid)
     RETURN_LONG(PHPCoroutine::get_cid());
 }
 
+static PHP_METHOD(swoole_coroutine_util, getPcid)
+{
+    RETURN_LONG(PHPCoroutine::get_pcid());
+}
+
 int php_coroutine_reactor_can_exit(swReactor *reactor)
 {
     return Coroutine::count() == 0;
@@ -495,25 +515,16 @@ static PHP_METHOD(swoole_coroutine_util, sleep)
     PHPCoroutine::check();
 
     double seconds;
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "d", & seconds) == FAILURE)
-    {
-        RETURN_FALSE;
-    }
 
-    int ms = (int) (seconds * 1000);
+    ZEND_PARSE_PARAMETERS_START(1, 1)
+        Z_PARAM_DOUBLE(seconds)
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    if (SwooleG.serv && swIsMaster())
-    {
-        swoole_php_fatal_error(E_WARNING, "cannot use timer in master process.");
-        return;
-    }
-    if (ms <= 0)
+    if (seconds <= 0)
     {
         swoole_php_fatal_error(E_WARNING, "Timer must be greater than 0");
-        return;
+        RETURN_FALSE;
     }
-
-    php_swoole_check_reactor();
     Coroutine::sleep(seconds);
     RETURN_TRUE;
 }
@@ -521,26 +532,26 @@ static PHP_METHOD(swoole_coroutine_util, sleep)
 static void aio_onReadCompleted(swAio_event *event)
 {
     zval *retval = NULL;
-    zval *result = NULL;
-    SW_MAKE_STD_ZVAL(result);
+    zval result;
 
     if (event->error == 0)
     {
-        ZVAL_STRINGL(result, (char* )event->buf, event->ret);
+        // TODO: Optimization: reduce memory copy
+        ZVAL_STRINGL(&result, (char* )event->buf, event->ret);
     }
     else
     {
         SwooleG.error = event->error;
-        ZVAL_BOOL(result, 0);
+        ZVAL_BOOL(&result, 0);
     }
 
     php_coro_context *context = (php_coro_context *) event->object;
-    int ret = PHPCoroutine::resume_m(context, result, retval);
+    int ret = PHPCoroutine::resume_m(context, &result, retval);
     if (ret == SW_CORO_ERR_END && retval)
     {
         zval_ptr_dtor(retval);
     }
-    zval_ptr_dtor(result);
+    zval_ptr_dtor(&result);
     efree(event->buf);
     efree(context);
 }
@@ -548,17 +559,16 @@ static void aio_onReadCompleted(swAio_event *event)
 static void aio_onFgetsCompleted(swAio_event *event)
 {
     zval *retval = NULL;
-    zval *result = NULL;
-    SW_MAKE_STD_ZVAL(result);
+    zval result;
 
     if (event->ret != -1)
     {
-        ZVAL_STRING(result, (char* )event->buf);
+        ZVAL_STRING(&result, (char* )event->buf);
     }
     else
     {
         SwooleG.error = event->error;
-        ZVAL_BOOL(result, 0);
+        ZVAL_BOOL(&result, 0);
     }
 
     php_coro_context *context = (php_coro_context *) event->object;
@@ -570,38 +580,36 @@ static void aio_onFgetsCompleted(swAio_event *event)
         stream->eof = 1;
     }
 
-    int ret = PHPCoroutine::resume_m(context, result, retval);
+    int ret = PHPCoroutine::resume_m(context, &result, retval);
     if (ret == SW_CORO_ERR_END && retval)
     {
         zval_ptr_dtor(retval);
     }
-    zval_ptr_dtor(result);
+    zval_ptr_dtor(&result);
     efree(context);
 }
 
 static void aio_onWriteCompleted(swAio_event *event)
 {
     zval *retval = NULL;
-    zval *result = NULL;
+    zval result;
 
-    SW_MAKE_STD_ZVAL(result);
     if (event->ret < 0)
     {
         SwooleG.error = event->error;
-        ZVAL_BOOL(result, 0);
+        ZVAL_BOOL(&result, 0);
     }
     else
     {
-        ZVAL_LONG(result, event->ret);
+        ZVAL_LONG(&result, event->ret);
     }
 
     php_coro_context *context = (php_coro_context *) event->object;
-    int ret = PHPCoroutine::resume_m(context, result, retval);
+    int ret = PHPCoroutine::resume_m(context, &result, retval);
     if (ret == SW_CORO_ERR_END && retval)
     {
         zval_ptr_dtor(retval);
     }
-    zval_ptr_dtor(result);
     efree(event->buf);
     efree(context);
 }
@@ -622,7 +630,7 @@ static int co_socket_onReadable(swReactor *reactor, swEvent *event)
         sock->timer = NULL;
     }
 
-    int n = read(sock->fd, sock->buf->val, sock->nbytes);
+    int n = read(sock->fd, ZSTR_VAL(sock->buf), sock->nbytes);
     if (n < 0)
     {
         ZVAL_FALSE(&result);
@@ -635,8 +643,8 @@ static int co_socket_onReadable(swReactor *reactor, swEvent *event)
     }
     else
     {
-        sock->buf->val[n] = 0;
-        sock->buf->len = n;
+        ZSTR_VAL(sock->buf)[n] = 0;
+        ZSTR_LEN(sock->buf) = n;
         ZVAL_STR(&result, sock->buf);
     }
     int ret = PHPCoroutine::resume_m(context, &result, retval);
@@ -676,7 +684,6 @@ static int co_socket_onWritable(swReactor *reactor, swEvent *event)
         ZVAL_LONG(&result, n);
     }
     int ret = PHPCoroutine::resume_m(context, &result, retval);
-    zval_ptr_dtor(&result);
     if (ret == SW_CORO_ERR_END && retval)
     {
         zval_ptr_dtor(retval);
@@ -1057,43 +1064,12 @@ static PHP_METHOD(swoole_coroutine_util, writeFile)
     }
 }
 
-static void coro_dns_onResolveCompleted(swAio_event *event)
-{
-    php_coro_context *context = (php_coro_context *) event->object;
-
-    zval *retval = NULL;
-    zval *result = NULL;
-
-    SW_MAKE_STD_ZVAL(result);
-
-    if (event->error == 0)
-    {
-        ZVAL_STRING(result, (char * )event->buf);
-    }
-    else
-    {
-        SwooleG.error = event->error;
-        ZVAL_BOOL(result, 0);
-    }
-
-    int ret = PHPCoroutine::resume_m(context, result, retval);
-    if (ret == SW_CORO_ERR_END && retval)
-    {
-        zval_ptr_dtor(retval);
-    }
-    zval_ptr_dtor(result);
-    efree(event->buf);
-    efree(context);
-}
-
 static void coro_dns_onGetaddrinfoCompleted(swAio_event *event)
 {
     php_coro_context *context = (php_coro_context *) event->object;
 
     zval *retval = NULL;
-    zval *result = NULL;
-
-    SW_MAKE_STD_ZVAL(result);
+    zval result;
 
     struct sockaddr_in *addr_v4;
     struct sockaddr_in6 *addr_v6;
@@ -1102,7 +1078,7 @@ static void coro_dns_onGetaddrinfoCompleted(swAio_event *event)
 
     if (req->error == 0)
     {
-        array_init(result);
+        array_init(&result);
         int i;
         char tmp[INET6_ADDRSTRLEN];
         const char *r ;
@@ -1121,22 +1097,22 @@ static void coro_dns_onGetaddrinfoCompleted(swAio_event *event)
             }
             if (r)
             {
-                add_next_index_string(result, tmp);
+                add_next_index_string(&result, tmp);
             }
         }
     }
     else
     {
-        ZVAL_BOOL(result, 0);
+        ZVAL_BOOL(&result, 0);
         SwooleG.error = req->error;
     }
 
-    int ret = PHPCoroutine::resume_m(context, result, retval);
+    int ret = PHPCoroutine::resume_m(context, &result, retval);
     if (ret == SW_CORO_ERR_END && retval)
     {
         zval_ptr_dtor(retval);
     }
-    zval_ptr_dtor(result);
+    zval_ptr_dtor(&result);
     efree(req->hostname);
     efree(req->result);
     if (req->service)
@@ -1258,22 +1234,25 @@ static PHP_METHOD(swoole_coroutine_util, getaddrinfo)
 
 static PHP_METHOD(swoole_coroutine_util, getBackTrace)
 {
-    zend_long cid;
+    zend_long cid = 0;
     zend_long options = DEBUG_BACKTRACE_PROVIDE_OBJECT;
     zend_long limit = 0;
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "l|ll", &cid, &options, &limit) == FAILURE)
-    {
-        RETURN_FALSE;
-    }
-    if (cid == PHPCoroutine::get_cid())
+    ZEND_PARSE_PARAMETERS_START(0, 3)
+        Z_PARAM_OPTIONAL
+        Z_PARAM_LONG(cid)
+        Z_PARAM_LONG(options)
+        Z_PARAM_LONG(limit)
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+
+    if (!cid || cid == PHPCoroutine::get_cid())
     {
         zend_fetch_debug_backtrace(return_value, 0, options, limit);
     }
     else
     {
-        php_coro_task *task = (php_coro_task *) Coroutine::get_task_by_cid(cid);
-        if (task == NULL)
+        php_coro_task *task = (php_coro_task *) PHPCoroutine::get_task_by_cid(cid);
+        if (UNEXPECTED(!task))
         {
             RETURN_FALSE;
         }
@@ -1329,7 +1308,7 @@ static PHP_METHOD(swoole_coroutine_iterator, __destruct)
     swoole_set_object(getThis(), NULL);
 }
 
-static PHP_METHOD(swoole_coroutine_util, listCoroutines)
+static PHP_METHOD(swoole_coroutine_util, list)
 {
     object_init_ex(return_value, swoole_coroutine_iterator_ce_ptr);
     coroutine_iterator *itearator = (coroutine_iterator *) emalloc(sizeof(coroutine_iterator));
@@ -1422,15 +1401,14 @@ PHP_FUNCTION(swoole_coroutine_exec)
         }
     }
 
-    zval *zdata;
-    SW_MAKE_STD_ZVAL(zdata);
+    zval zdata;
     if (buffer->length == 0)
     {
-        ZVAL_EMPTY_STRING(zdata);
+        ZVAL_EMPTY_STRING(&zdata);
     }
     else
     {
-        ZVAL_STRINGL(zdata, buffer->str, buffer->length);
+        ZVAL_STRINGL(&zdata, buffer->str, buffer->length);
     }
 
     int status;
@@ -1440,11 +1418,11 @@ PHP_FUNCTION(swoole_coroutine_exec)
         array_init(return_value);
         add_assoc_long(return_value, "code", WEXITSTATUS(status));
         add_assoc_long(return_value, "signal", WTERMSIG(status));
-        add_assoc_zval(return_value, "output", zdata);
+        add_assoc_zval(return_value, "output", &zdata);
     }
     else
     {
-        zval_ptr_dtor(zdata);
+        zval_ptr_dtor(&zdata);
         RETVAL_FALSE;
     }
 
@@ -1453,16 +1431,16 @@ PHP_FUNCTION(swoole_coroutine_exec)
 
 static void coro_onDefer(void *data)
 {
-    php_defer_fci *defer_fci = (php_defer_fci *) data;
-    zval _retval, *retval = &_retval;
+    php_swoole_fci *defer_fci = (php_swoole_fci *) data;
+    zval retval;
 
-    defer_fci->fci.retval = retval;
+    defer_fci->fci.retval = &retval;
     if (sw_call_function_anyway(&defer_fci->fci, &defer_fci->fci_cache) == FAILURE)
     {
         swoole_php_fatal_error(E_WARNING, "defer callback handler error.");
         return;
     }
-    zval_ptr_dtor(retval);
+    zval_ptr_dtor(&retval);
     sw_fci_cache_discard(&defer_fci->fci_cache);
     efree(defer_fci);
 }
@@ -1471,16 +1449,15 @@ PHP_FUNCTION(swoole_coroutine_defer)
 {
     zend_fcall_info fci = empty_fcall_info;
     zend_fcall_info_cache fci_cache = empty_fcall_info_cache;
-    php_defer_fci *defer_fci;
+    php_swoole_fci *defer_fci;
 
     PHPCoroutine::check();
 
-    ZEND_PARSE_PARAMETERS_START(1, -1)
+    ZEND_PARSE_PARAMETERS_START(1, 1)
         Z_PARAM_FUNC(fci, fci_cache)
-        Z_PARAM_VARIADIC('*', fci.params, fci.param_count)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    defer_fci = (php_defer_fci *) emalloc(sizeof(php_defer_fci));
+    defer_fci = (php_swoole_fci *) emalloc(sizeof(php_swoole_fci));
     defer_fci->fci = fci;
     defer_fci->fci_cache = fci_cache;
     sw_fci_cache_persist(&defer_fci->fci_cache);
