@@ -16,7 +16,7 @@
  +----------------------------------------------------------------------+
  */
 
-#include "php_swoole.h"
+#include "php_swoole_cxx.h"
 
 #include "swoole_http_client.h"
 #include "swoole_coroutine.h"
@@ -48,10 +48,70 @@ static const swoole_http_parser_settings http_parser_settings =
     http_parser_on_message_complete
 };
 
+class http_socket: public Socket
+{
+public:
+    http_socket(swSocket_type socket_type) :
+            Socket(socket_type)
+    {
+    }
+    bool recv_http_response(const swoole_http_parser_settings *settings, void *data, double timeout = 0)
+    {
+        ssize_t retval = 0;
+        size_t total_bytes = 0, parsed_n = 0;
+        swString *buffer = get_read_buffer();
+        timer timer(&read_timer, timeout == 0 ? this->timeout : timeout, this, timer_callback);
+        if (unlikely(!timer()))
+        {
+            return false;
+        }
+        // re-init http response parser
+        swoole_http_parser_init(&parser, PHP_HTTP_RESPONSE);
+        parser.data = data;
+        do
+        {
+            if (unlikely(parser.state == s_dead))
+            {
+                set_err(EPROTO);
+                return false;
+            }
+            retval = recv(buffer->str, buffer->size);
+            if (retval > 0)
+            {
+                total_bytes += retval;
+                parsed_n = swoole_http_parser_execute(&parser, settings, buffer->str, retval);
+                swTraceLog(SW_TRACE_HTTP_CLIENT, "parsed_n=%ld, retval=%ld, total_bytes=%ld, completed=%d.", parsed_n, retval, total_bytes, parser.state != s_start_res);
+            }
+            else
+            {
+                if (retval == 0)
+                {
+                    set_err(ECONNRESET);
+                    if (total_bytes > 0 && !swoole_http_should_keep_alive(&parser))
+                    {
+                        http_parser_on_message_complete(&parser);
+                        return true;
+                    }
+                }
+                return false;
+            }
+        } while (parser.state != s_start_res);
+        // websocket stick package
+        if (parser.upgrade && (size_t) retval > parsed_n + 1 + SW_WEBSOCKET_HEADER_LEN)
+        {
+            buffer->length = retval - parsed_n - 1;
+            memmove(buffer->str, buffer->str + parsed_n + 1, buffer->length);
+        }
+        return true;
+    }
+private:
+    swoole_http_parser parser = {0};
+};
+
 class http_client
 {
     private:
-    Socket* socket = nullptr;
+    http_socket* socket = nullptr;
     swSocket_type socket_type = SW_SOCK_TCP;
 
     public:
@@ -72,7 +132,6 @@ class http_client
     std::string uri;
 
     /* response parse */
-    swoole_http_parser parser = {0};
     char *tmp_header_field_name = nullptr;
     int tmp_header_field_name_len = 0;
     swString *body = nullptr;
@@ -89,7 +148,6 @@ class http_client
     bool websocket = false;            // if upgrade successfully
     bool gzip = false;               // enable gzip
     bool chunked = false;            // Transfer-Encoding: chunked
-    bool completed = false;          // response parse over
     bool websocket_mask = false;     // enable websocket mask
     bool is_download = false;        // save http response to file
     int download_file_fd = 0;
@@ -281,7 +339,7 @@ static int http_parser_on_header_value(swoole_http_parser *parser, const char *a
 
     add_assoc_stringl_ex(zheaders, header_name, http->tmp_header_field_name_len, (char *) at, length);
 
-    if (http->parser.status_code == SW_HTTP_SWITCHING_PROTOCOLS && strcmp(header_name, "upgrade") == 0 && strncasecmp(at, "websocket", length) == 0)
+    if (parser->status_code == SW_HTTP_SWITCHING_PROTOCOLS && strcmp(header_name, "upgrade") == 0 && strncasecmp(at, "websocket", length) == 0)
     {
         http->websocket = true;
     }
@@ -317,7 +375,6 @@ static int http_parser_on_headers_complete(swoole_http_parser *parser)
     if (http->chunked == 0 && parser->content_length == -1)
     {
         enum flags { F_CONNECTION_CLOSE = 1 << 2 };
-        http->state = HTTP_CLIENT_STATE_WAIT_CLOSE;
         parser->flags |= F_CONNECTION_CLOSE;
     }
     if (http->method == SW_HTTP_HEAD || parser->status_code == SW_HTTP_NO_CONTENT)
@@ -374,12 +431,8 @@ static int http_parser_on_message_complete(swoole_http_parser *parser)
     }
 
 #ifdef SW_HAVE_ZLIB
-    if (http->gzip && http->body->length > 0)
+    if (http->gzip && http->body->length > 0 && http->uncompress_response())
     {
-        if (!http->uncompress_response())
-        {
-            return 0;
-        }
         zend_update_property_stringl(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("body"), SW_STRINGL(http->gzip_buffer));
     }
     else
@@ -388,10 +441,8 @@ static int http_parser_on_message_complete(swoole_http_parser *parser)
         zend_update_property_stringl(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("body"), SW_STRINGL(http->body));
     }
 
-    http->completed = 1;
-
     //http status code
-    zend_update_property_long(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("statusCode"), http->parser.status_code);
+    zend_update_property_long(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("statusCode"), parser->status_code);
 
     if (parser->upgrade)
     {
@@ -409,7 +460,7 @@ http_client::http_client(zval* zobject, std::string host, zend_long port, zend_b
     // check host
     if (host.length() == 0)
     {
-        swoole_php_fatal_error(E_ERROR, "host is empty.");
+        swoole_php_fatal_error(E_WARNING, "host is empty.");
         return;
     }
     if (host.compare(0, 6, "unix:/", 0, 6) == 0)
@@ -426,14 +477,6 @@ http_client::http_client(zval* zobject, std::string host, zend_long port, zend_b
     {
         socket_type = SW_SOCK_TCP;
     }
-    if (socket_type == SW_SOCK_TCP || socket_type == SW_SOCK_TCP6)
-    {
-        if (port <= 0 || port > SW_CLIENT_MAX_PORT)
-        {
-            swoole_php_fatal_error(E_ERROR, "The port " ZEND_LONG_FMT " is invaild.", port);
-            return;
-        }
-    }
     this->host = host;
     this->port = port;
 #ifdef SW_USE_OPENSSL
@@ -441,15 +484,11 @@ http_client::http_client(zval* zobject, std::string host, zend_long port, zend_b
 #else
     if (ssl)
     {
-        swoole_php_fatal_error(E_ERROR, "need to use `--enable-openssl` to support ssl when compiling swoole.");
+        swoole_php_fatal_error(E_ERROR, "Need to use `--enable-openssl` to support ssl when compiling swoole.");
         return;
     }
 #endif
 
-    // init http response parser
-    swoole_http_parser_init(&parser, PHP_HTTP_RESPONSE);
-    parser.data = this;
-
     _zobject = *zobject;
     // TODO: zend_read_property cache here (strong type properties)
 }
@@ -553,7 +592,7 @@ void http_client::check_bind()
 {
     if (socket)
     {
-        PHPCoroutine::check_bind("http client", socket->has_bound());
+        PHPCoroutine::check_bind("http client", socket->get_bound_cid());
     }
 }
 
@@ -571,10 +610,8 @@ void http_client::set(zval *zset = nullptr)
         vht = Z_ARRVAL_P(zset);
         if (php_swoole_array_get_value(vht, "timeout", ztmp))
         {
-            convert_to_double(ztmp);
             // backward compatibility
-            connect_timeout = (double) Z_DVAL_P(ztmp);
-            timeout = (double) Z_DVAL_P(ztmp);
+            timeout = connect_timeout = zval_get_double(ztmp);
             if (socket)
             {
                 socket->set_timeout(timeout);
@@ -582,28 +619,23 @@ void http_client::set(zval *zset = nullptr)
         }
         if (php_swoole_array_get_value(vht, "connect_timeout", ztmp))
         {
-            convert_to_double(ztmp);
-            connect_timeout = (double) Z_DVAL_P(ztmp);
+            connect_timeout = zval_get_double(ztmp);
         }
         if (php_swoole_array_get_value(vht, "reconnect", ztmp))
         {
-            convert_to_long(ztmp);
-            reconnect_interval = (uint8_t) MIN(Z_LVAL_P(ztmp), UINT8_MAX);
+            reconnect_interval = (uint8_t) MIN(zval_get_long(ztmp), UINT8_MAX);
         }
         if (php_swoole_array_get_value(vht, "defer", ztmp))
         {
-            convert_to_boolean(ztmp);
-            defer = Z_BVAL_P(ztmp);
+            defer = zval_is_true(ztmp);
         }
         if (php_swoole_array_get_value(vht, "keep_alive", ztmp))
         {
-            convert_to_boolean(ztmp);
-            keep_alive = Z_BVAL_P(ztmp);
+            keep_alive = zval_is_true(ztmp);
         }
         if (php_swoole_array_get_value(vht, "websocket_mask", ztmp))
         {
-            convert_to_boolean(ztmp);
-            websocket_mask = Z_BVAL_P(ztmp);
+            websocket_mask = zval_is_true(ztmp);
         }
     }
     if (socket)
@@ -616,7 +648,7 @@ bool http_client::connect()
 {
     if (!socket)
     {
-        socket = new Socket(socket_type);
+        socket = new http_socket(socket_type);
         if (unlikely(socket->socket == nullptr))
         {
             swoole_php_fatal_error(E_WARNING, "new Socket() failed. Error: %s [%d]", strerror(errno), errno);
@@ -671,8 +703,8 @@ bool http_client::keep_liveness()
             zend_update_property_long(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("errCode"), socket->errCode);
             zend_update_property_string(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("errMsg"), socket->errMsg);
             zend_update_property_long(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("statusCode"), HTTP_CLIENT_ESTATUS_SERVER_RESET);
+            close();
         }
-        close();
         for (; reconnected_count < reconnect_interval; reconnected_count++)
         {
             if (connect())
@@ -696,7 +728,7 @@ bool http_client::send()
 
     if (uri.length() == 0)
     {
-        swoole_php_fatal_error(E_ERROR, "path is empty.");
+        swoole_php_fatal_error(E_WARNING, "path is empty.");
         return false;
     }
 
@@ -744,8 +776,8 @@ bool http_client::send()
     // ============ download ============
     if (z_download_file)
     {
-        convert_to_string(z_download_file);
-        char *download_file_name = Z_STRVAL_P(z_download_file);
+        zend::string str_download_file(z_download_file);
+        char *download_file_name = str_download_file.val();
         zval *z_download_offset = sw_zend_read_property_not_null(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("downloadOffset"), 1);
         off_t download_offset = 0;
         if (z_download_offset)
@@ -782,15 +814,17 @@ bool http_client::send()
     }
 
     // ============ method ============
+    zend::string str_method;
     if (zmethod)
     {
-        convert_to_string(zmethod);
-        method = Z_STRVAL_P(zmethod);
+        str_method = zmethod;
+        method = str_method.val();
     }
     else
     {
         method = (char *) (zbody ? "POST" : "GET");
     }
+
     this->method = swHttp_get_method(method, strlen(method) + 1);
     swString_append_ptr(http_client_buffer, method, strlen(method));
     swString_append_ptr(http_client_buffer, ZEND_STRL(" "));
@@ -802,6 +836,7 @@ bool http_client::send()
     if (socket->http_proxy)
 #endif
     {
+        zend::string str_host;
         const static char *pre = "http://";
         char *_host = (char *) host.c_str();
         size_t _host_len = host.length();
@@ -809,13 +844,14 @@ bool http_client::send()
         {
             if ((value = zend_hash_str_find(Z_ARRVAL_P(zheaders), ZEND_STRL("Host"))))
             {
-                _host = Z_STRVAL_P(value);
-                _host_len = Z_STRLEN_P(value);
+                str_host = value;
+                _host = str_host.val();
+                _host_len = str_host.len();
             }
         }
         size_t proxy_uri_len = uri.length() + _host_len + strlen(pre) + 10;
         char *proxy_uri = (char*) emalloc(proxy_uri_len);
-        proxy_uri_len = snprintf(proxy_uri, proxy_uri_len, "%s%s:%u%s", pre, _host, port, uri.c_str());
+        proxy_uri_len = sw_snprintf(proxy_uri, proxy_uri_len, "%s%s:%u%s", pre, _host, port, uri.c_str());
         uri = std::string(proxy_uri, proxy_uri_len);
         efree(proxy_uri);
     }
@@ -847,8 +883,8 @@ bool http_client::send()
             {
                 continue;
             }
-            convert_to_string(value);
-            if ((Z_STRLEN_P(value) == 0) || (strncasecmp(key, ZEND_STRL("Host")) == 0))
+            zend::string _value(value);
+            if ((_value.len() == 0) || (strncasecmp(key, ZEND_STRL("Host")) == 0))
             {
                 continue;
             }
@@ -866,7 +902,7 @@ bool http_client::send()
             {
                 header_flag |= HTTP_HEADER_ACCEPT_ENCODING;
             }
-            http_client_swString_append_headers(http_client_buffer, key, keylen, Z_STRVAL_P(value), Z_STRLEN_P(value));
+            http_client_swString_append_headers(http_client_buffer, key, keylen, _value.val(), _value.len());
         SW_HASHTABLE_FOREACH_END();
     }
     else
@@ -905,8 +941,8 @@ bool http_client::send()
             {
                 continue;
             }
-            convert_to_string(value);
-            if (Z_STRLEN_P(value) == 0)
+            zend::string _value(value);
+            if (_value.len() == 0)
             {
                 continue;
             }
@@ -914,7 +950,7 @@ bool http_client::send()
             swString_append_ptr(http_client_buffer, "=", 1);
 
             int encoded_value_len;
-            encoded_value = sw_php_url_encode(Z_STRVAL_P(value), Z_STRLEN_P(value), &encoded_value_len);
+            encoded_value = sw_php_url_encode(_value.val(), _value.len(), &encoded_value_len);
             if (encoded_value)
             {
                 swString_append_ptr(http_client_buffer, encoded_value, encoded_value_len);
@@ -943,7 +979,7 @@ bool http_client::send()
             boundary_str + sizeof(SW_HTTP_CLIENT_BOUNDARY_PREKEY) - 1,
             sizeof(boundary_str) - sizeof(SW_HTTP_CLIENT_BOUNDARY_PREKEY)
         );
-        n = snprintf(
+        n = sw_snprintf(
             header_buf,
             sizeof(header_buf), "Content-Type: multipart/form-data; boundary=%*s\r\n",
             (int)(sizeof(boundary_str) - 1), boundary_str
@@ -961,10 +997,10 @@ bool http_client::send()
                 {
                     continue;
                 }
-                convert_to_string(value);
+                zend::string _value(value);
                 //strlen("%.*")*2 = 6
                 //header + body + CRLF
-                content_length += (sizeof(SW_HTTP_FORM_DATA_FORMAT_STRING) - 7) + (sizeof(boundary_str) - 1) + keylen + Z_STRLEN_P(value) + 2;
+                content_length += (sizeof(SW_HTTP_FORM_DATA_FORMAT_STRING) - 7) + (sizeof(boundary_str) - 1) + keylen + _value.len() + 2;
             SW_HASHTABLE_FOREACH_END();
         }
 
@@ -1013,14 +1049,14 @@ bool http_client::send()
                 {
                     continue;
                 }
-                convert_to_string(value);
-                n = snprintf(
+                zend::string _value(value);
+                n = sw_snprintf(
                     header_buf, sizeof(header_buf),
                     SW_HTTP_FORM_DATA_FORMAT_STRING, (int)(sizeof(boundary_str) - 1),
                     boundary_str, keylen, key
                 );
                 swString_append_ptr(http_client_buffer, header_buf, n);
-                swString_append_ptr(http_client_buffer, Z_STRVAL_P(value), Z_STRLEN_P(value));
+                swString_append_ptr(http_client_buffer, _value.val(), _value.len());
                 swString_append_ptr(http_client_buffer, ZEND_STRL("\r\n"));
             SW_HASHTABLE_FOREACH_END();
         }
@@ -1074,7 +1110,7 @@ bool http_client::send()
                 /**
                  * part header
                  */
-                n = snprintf(
+                n = sw_snprintf(
                     header_buf, sizeof(header_buf), SW_HTTP_FORM_DATA_FORMAT_FILE,
                     (int) (sizeof(boundary_str) - 1), boundary_str,
                     (int) Z_STRLEN_P(zname), Z_STRVAL_P(zname),
@@ -1116,7 +1152,7 @@ bool http_client::send()
             SW_HASHTABLE_FOREACH_END();
         }
 
-        n = snprintf(header_buf, sizeof(header_buf), "--%*s--\r\n", (int)(sizeof(boundary_str) - 1), boundary_str);
+        n = sw_snprintf(header_buf, sizeof(header_buf), "--%*s--\r\n", (int)(sizeof(boundary_str) - 1), boundary_str);
         if (socket->send(header_buf, n) < 0)
         {
             goto _send_fail;
@@ -1206,10 +1242,6 @@ bool http_client::exec(std::string uri)
 
 bool http_client::recv(double timeout)
 {
-    long parsed_n = 0;
-    ssize_t total_bytes = 0, retval = 0;
-    swString *buffer;
-
     check_bind();
 
     if (!wait)
@@ -1223,74 +1255,32 @@ bool http_client::recv(double timeout)
         zend_update_property_long(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("statusCode"), HTTP_CLIENT_ESTATUS_SERVER_RESET);
         return false;
     }
-
-    buffer = socket->get_read_buffer();
-    socket->set_timer(Socket::TIMER_LV_GLOBAL, timeout);
-    while (completed == 0)
+    if (!socket->recv_http_response(&http_parser_settings, this, timeout == 0 ? this->timeout : timeout))
     {
-        retval = socket->recv(buffer->str, buffer->size);
-        if (retval > 0)
+        zend_update_property_long(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("errCode"), socket->errCode);
+        zend_update_property_string(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("errMsg"), socket->errMsg);
+        if (socket->errCode == ETIMEDOUT)
         {
-            total_bytes += retval;
-            parsed_n = swoole_http_parser_execute(&parser, &http_parser_settings, buffer->str, retval);
-            swTraceLog(SW_TRACE_HTTP_CLIENT, "parsed_n=%ld, retval=%ld, total_bytes=%ld, completed=%d.", parsed_n, retval, total_bytes, completed);
-            if (unlikely(parser.state == s_dead && !completed))
-            {
-                socket->set_err(EPROTO);
-                goto _error;
-            }
-            if (parsed_n >= 0)
-            {
-                continue;
-            }
-        }
-        else if (retval == 0 && state == HTTP_CLIENT_STATE_WAIT_CLOSE)
-        {
-            http_parser_on_message_complete(&parser);
-            break;
+            zend_update_property_long(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("statusCode"), HTTP_CLIENT_ESTATUS_REQUEST_TIMEOUT);
         }
         else
         {
-            _error:
-            // IO error
-            socket->del_timer(Socket::TIMER_LV_GLOBAL);
-            zend_update_property_long(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("errCode"), socket->errCode);
-            zend_update_property_string(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("errMsg"), socket->errMsg);
-            if (socket->errCode == ETIMEDOUT)
-            {
-                zend_update_property_long(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("statusCode"), HTTP_CLIENT_ESTATUS_REQUEST_TIMEOUT);
-            }
-            else
-            {
-                zend_update_property_long(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("statusCode"), HTTP_CLIENT_ESTATUS_SERVER_RESET);
-            }
-            close();
-            return false;
+            zend_update_property_long(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("statusCode"), HTTP_CLIENT_ESTATUS_SERVER_RESET);
         }
+        close();
+        return false;
     }
-    socket->del_timer(Socket::TIMER_LV_GLOBAL);
-
     /**
      * TODO: Sec-WebSocket-Accept check
      */
     if (websocket)
     {
-        state = HTTP_CLIENT_STATE_UPGRADE;
         socket->open_length_check = 1;
         socket->protocol.get_package_length = swWebSocket_get_package_length;
         socket->protocol.package_length_size = SW_WEBSOCKET_HEADER_LEN;
-        /**
-         * websocket message queue
-         */
-        if (retval > parsed_n + 3)
-        {
-            buffer->length = retval - parsed_n - 1;
-            memmove(buffer->str, buffer->str + parsed_n + 1, buffer->length);
-        }
     }
-
     // handler keep alive
-    if (!keep_alive && state != HTTP_CLIENT_STATE_WAIT_CLOSE && !websocket)
+    if (!keep_alive && !websocket)
     {
         close();
     }
@@ -1316,9 +1306,7 @@ void http_client::recv(zval *zframe, double timeout)
         return;
     }
 
-    socket->set_timer(Socket::TIMER_LV_GLOBAL, timeout);
-    ssize_t retval = socket->recv_packet();
-    socket->del_timer(Socket::TIMER_LV_GLOBAL);
+    ssize_t retval = socket->recv_packet(timeout);
     if (retval <= 0)
     {
         zend_update_property_long(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("errCode"), socket->errCode);
@@ -1399,8 +1387,6 @@ bool http_client::push(zval *zdata, zend_long opcode, bool fin)
 void http_client::reset()
 {
     wait = false;
-    state = HTTP_CLIENT_STATE_READY;
-    completed = false;
     // clear
 #ifdef SW_HAVE_ZLIB
     if (gzip)
@@ -1425,24 +1411,21 @@ void http_client::reset()
 
 bool http_client::close()
 {
-    bool ret;
-    if (!socket)
+    Socket *socket = this->socket;
+    if (socket)
     {
-        return false;
+        zend_update_property_bool(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("connected"), 0);
+        if (!socket->has_bound())
+        {
+            // reset some request period states
+            reset();
+            // reset the properties that depend on the connection
+            this->websocket = false;
+            this->socket = nullptr;
+        }
+        return php_swoole_client_coro_socket_free(socket);
     }
-    zend_update_property_bool(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("connected"), 0);
-
-    // reset some request period states
-    reset();
-
-    // reset the properties that depend on the connection
-    websocket = false;
-
-    // close socket
-    ret = php_swoole_client_coro_socket_free(socket);
-    socket = nullptr;
-
-    return ret;
+    return false;
 }
 
 http_client::~http_client()
@@ -1481,7 +1464,6 @@ static void swoole_http_client_coro_free_object(zend_object *object)
     http_client_coro *hcc_t = swoole_http_client_coro_fetch_object(object);
     if (hcc_t->phc)
     {
-        hcc_t->phc->close();
         delete hcc_t->phc;
         hcc_t->phc = nullptr;
     }
@@ -1727,18 +1709,16 @@ static PHP_METHOD(swoole_http_client_coro, addFile)
     }
 
     zval *zupload_files = sw_zend_read_property_array(swoole_http_client_coro_ce_ptr, getThis(), ZEND_STRL("uploadFiles"), 1);
-    zval *zupload_file;
-    SW_MAKE_STD_ZVAL(zupload_file);
-    array_init(zupload_file);
-    add_assoc_stringl_ex(zupload_file, ZEND_STRL("path"), path, l_path);
-    add_assoc_stringl_ex(zupload_file, ZEND_STRL("name"), name, l_name);
-    add_assoc_stringl_ex(zupload_file, ZEND_STRL("filename"), filename, l_filename);
-    add_assoc_stringl_ex(zupload_file, ZEND_STRL("type"), type, l_type);
-    add_assoc_long(zupload_file, "size", length);
-    add_assoc_long(zupload_file, "offset", offset);
-    add_next_index_zval(zupload_files, zupload_file);
+    zval zupload_file;
+    array_init(&zupload_file);
+    add_assoc_stringl_ex(&zupload_file, ZEND_STRL("path"), path, l_path);
+    add_assoc_stringl_ex(&zupload_file, ZEND_STRL("name"), name, l_name);
+    add_assoc_stringl_ex(&zupload_file, ZEND_STRL("filename"), filename, l_filename);
+    add_assoc_stringl_ex(&zupload_file, ZEND_STRL("type"), type, l_type);
+    add_assoc_long(&zupload_file, "size", length);
+    add_assoc_long(&zupload_file, "offset", offset);
 
-    RETURN_TRUE;
+    RETURN_BOOL(add_next_index_zval(zupload_files, &zupload_file) == SUCCESS);
 }
 
 static PHP_METHOD(swoole_http_client_coro, addData)
@@ -1772,17 +1752,15 @@ static PHP_METHOD(swoole_http_client_coro, addData)
     }
 
     zval *zupload_files = sw_zend_read_property_array(swoole_http_client_coro_ce_ptr, getThis(), ZEND_STRL("uploadFiles"), 1);
-    zval *zupload_file;
-    SW_MAKE_STD_ZVAL(zupload_file);
-    array_init(zupload_file);
-    add_assoc_stringl_ex(zupload_file, ZEND_STRL("content"), data, l_data);
-    add_assoc_stringl_ex(zupload_file, ZEND_STRL("name"), name, l_name);
-    add_assoc_stringl_ex(zupload_file, ZEND_STRL("filename"), filename, l_filename);
-    add_assoc_stringl_ex(zupload_file, ZEND_STRL("type"), type, l_type);
-    add_assoc_long(zupload_file, "size", l_data);
-    add_next_index_zval(zupload_files, zupload_file);
+    zval zupload_file;
+    array_init(&zupload_file);
+    add_assoc_stringl_ex(&zupload_file, ZEND_STRL("content"), data, l_data);
+    add_assoc_stringl_ex(&zupload_file, ZEND_STRL("name"), name, l_name);
+    add_assoc_stringl_ex(&zupload_file, ZEND_STRL("filename"), filename, l_filename);
+    add_assoc_stringl_ex(&zupload_file, ZEND_STRL("type"), type, l_type);
+    add_assoc_long(&zupload_file, "size", l_data);
 
-    RETURN_TRUE;
+    RETURN_BOOL(add_next_index_zval(zupload_files, &zupload_file) == SUCCESS);
 }
 
 static PHP_METHOD(swoole_http_client_coro, execute)
