@@ -14,7 +14,7 @@
   +----------------------------------------------------------------------+
 */
 
-#include "php_swoole.h"
+#include "php_swoole_cxx.h"
 #include "swoole_http.h"
 #include "swoole_coroutine.h"
 
@@ -708,13 +708,12 @@ static int multipart_body_on_header_value(multipart_parser* p, const char *at, s
             return SW_OK;
         }
 
-        zval *tmp_array;
-        SW_MAKE_STD_ZVAL(tmp_array);
-        array_init(tmp_array);
-        http_parse_cookie(tmp_array, (char *) at + sizeof("form-data;") - 1, length - sizeof("form-data;") + 1);
+        zval tmp_array;
+        array_init(&tmp_array);
+        http_parse_cookie(&tmp_array, (char *) at + sizeof("form-data;") - 1, length - sizeof("form-data;") + 1);
 
         zval *form_name;
-        if (!(form_name = zend_hash_str_find(Z_ARRVAL_P(tmp_array), ZEND_STRL("name"))))
+        if (!(form_name = zend_hash_str_find(Z_ARRVAL(tmp_array), ZEND_STRL("name"))))
         {
             return SW_OK;
         }
@@ -731,7 +730,7 @@ static int multipart_body_on_header_value(multipart_parser* p, const char *at, s
 
         zval *filename;
         //POST form data
-        if (!(filename = zend_hash_str_find(Z_ARRVAL_P(tmp_array), ZEND_STRL("filename"))))
+        if (!(filename = zend_hash_str_find(Z_ARRVAL(tmp_array), ZEND_STRL("filename"))))
         {
             ctx->current_form_data_name = estrndup(tmp, value_len);
             ctx->current_form_data_name_len = value_len;
@@ -763,7 +762,7 @@ static int multipart_body_on_header_value(multipart_parser* p, const char *at, s
 
             ctx->current_multipart_header = multipart_header;
         }
-        zval_ptr_dtor(tmp_array);
+        zval_ptr_dtor(&tmp_array);
     }
 
     if (strncasecmp(headername, "content-type", header_len) == 0 && ctx->current_multipart_header)
@@ -1044,7 +1043,7 @@ int php_swoole_http_onReceive(swServer *serv, swEventData *req)
         swWarn("swoole_http_parser_execute failed.");
         if (conn->websocket_status == WEBSOCKET_STATUS_CONNECTION)
         {
-            return swServer_tcp_close(serv, fd, 1);
+            return serv->close(serv, fd, 1);
         }
     }
     else
@@ -1116,7 +1115,7 @@ int php_swoole_http_onReceive(swServer *serv, swEventData *req)
             if (PHPCoroutine::create(fci_cache, 2, args) < 0)
             {
                 swoole_php_error(E_WARNING, "create Http onRequest coroutine error.");
-                swServer_tcp_close(serv, fd, 0);
+                serv->close(serv, fd, 0);
             }
         }
         else
@@ -1496,12 +1495,14 @@ static PHP_METHOD(swoole_http_response, write)
     ctx->enable_compression = 0;
 #endif
 
+    swServer *serv = SwooleG.serv;
+
     if (!ctx->send_header)
     {
         ctx->chunk = 1;
         swString_clear(swoole_http_buffer);
         http_build_header(ctx, getThis(), swoole_http_buffer, -1);
-        if (swServer_tcp_send(SwooleG.serv, ctx->fd, swoole_http_buffer->str, swoole_http_buffer->length) < 0)
+        if (serv->send(serv, ctx->fd, swoole_http_buffer->str, swoole_http_buffer->length) < 0)
         {
             ctx->chunk = 0;
             ctx->send_header = 0;
@@ -1538,8 +1539,7 @@ static PHP_METHOD(swoole_http_response, write)
     swString_append_ptr(swoole_http_buffer, ZEND_STRL("\r\n"));
     sw_free(hex_string);
 
-    swServer *serv = SwooleG.serv;
-    int ret = swServer_tcp_send(serv, ctx->fd, swoole_http_buffer->str, swoole_http_buffer->length);
+    int ret = serv->send(serv, ctx->fd, swoole_http_buffer->str, swoole_http_buffer->length);
 #ifdef SW_COROUTINE
     if (ret < 0 && SwooleG.error == SW_ERROR_OUTPUT_BUFFER_OVERFLOW && serv->send_yield)
     {
@@ -1589,11 +1589,11 @@ static void http_build_header(http_context *ctx, zval *zobject, swString *respon
      */
     if (!ctx->response.reason)
     {
-        n = snprintf(buf, l_buf, "HTTP/1.1 %s\r\n", http_status_message(ctx->response.status));
+        n = sw_snprintf(buf, l_buf, "HTTP/1.1 %s\r\n", http_status_message(ctx->response.status));
     }
     else
     {
-        n = snprintf(buf, l_buf, "HTTP/1.1 %d %s\r\n", ctx->response.status, ctx->response.reason);
+        n = sw_snprintf(buf, l_buf, "HTTP/1.1 %d %s\r\n", ctx->response.status, ctx->response.reason);
     }
     swString_append_ptr(response, buf, n);
 
@@ -1615,7 +1615,7 @@ static void http_build_header(http_context *ctx, zval *zobject, swString *respon
             // TODO: numeric key name neccessary?
             if (!key)
             {
-                break;
+                continue;
             }
             if (strncasecmp(key, "Server", keylen) == 0)
             {
@@ -1643,8 +1643,8 @@ static void http_build_header(http_context *ctx, zval *zobject, swString *respon
             }
             if (!ZVAL_IS_NULL(value))
             {
-                convert_to_string(value);
-                n = snprintf(buf, l_buf, "%*s: %*s\r\n", keylen - 1, key, (int)Z_STRLEN_P(value), Z_STRVAL_P(value));
+                zend::string str_value(value);
+                n = sw_snprintf(buf, l_buf, "%*s: %*s\r\n", keylen - 1, key, (int) str_value.len(), str_value.val());
                 swString_append_ptr(response, buf, n);
             }
         }
@@ -1681,7 +1681,7 @@ static void http_build_header(http_context *ctx, zval *zobject, swString *respon
     if (!(header_flag & HTTP_HEADER_DATE))
     {
         date_str = sw_php_format_date((char *) ZEND_STRL(SW_HTTP_DATE_FORMAT), serv->gs->now, 0);
-        n = snprintf(buf, l_buf, "Date: %s\r\n", date_str);
+        n = sw_snprintf(buf, l_buf, "Date: %s\r\n", date_str);
         swString_append_ptr(response, buf, n);
         efree(date_str);
     }
@@ -1702,7 +1702,7 @@ static void http_build_header(http_context *ctx, zval *zobject, swString *respon
             body_length = swoole_zlib_buffer->length;
         }
 #endif
-        n = snprintf(buf, l_buf, "Content-Length: %d\r\n", body_length);
+        n = sw_snprintf(buf, l_buf, "Content-Length: %d\r\n", body_length);
         swString_append_ptr(response, buf, n);
     }
 
@@ -1964,9 +1964,11 @@ static PHP_METHOD(swoole_http_response, end)
     }
 #endif
 
+    swServer *serv = SwooleG.serv;
+
     if (ctx->chunk)
     {
-        ret = swServer_tcp_send(SwooleG.serv, ctx->fd, (char*) ZEND_STRL("0\r\n\r\n"));
+        ret = serv->send(serv, ctx->fd, (char*) ZEND_STRL("0\r\n\r\n"));
         if (ret < 0)
         {
             RETURN_FALSE;
@@ -2001,7 +2003,7 @@ static PHP_METHOD(swoole_http_response, end)
             }
         }
 
-        ret = swServer_tcp_send(SwooleG.serv, ctx->fd, swoole_http_buffer->str, swoole_http_buffer->length);
+        ret = serv->send(serv, ctx->fd, swoole_http_buffer->str, swoole_http_buffer->length);
         if (ret < 0)
         {
             ctx->send_header = 0;
@@ -2011,7 +2013,7 @@ static PHP_METHOD(swoole_http_response, end)
 
     if (ctx->upgrade)
     {
-        swConnection *conn = swWorker_get_connection(SwooleG.serv, ctx->fd);
+        swConnection *conn = swWorker_get_connection(serv, ctx->fd);
         if (conn && conn->websocket_status == WEBSOCKET_STATUS_HANDSHAKE && ctx->response.status == 101)
         {
             conn->websocket_status = WEBSOCKET_STATUS_ACTIVE;
@@ -2020,7 +2022,7 @@ static PHP_METHOD(swoole_http_response, end)
 
     if (!ctx->keepalive)
     {
-        swServer_tcp_close(SwooleG.serv, ctx->fd, 0);
+        serv->close(serv, ctx->fd, 0);
     }
     swoole_http_context_free(ctx);
     RETURN_TRUE;
@@ -2072,12 +2074,12 @@ static PHP_METHOD(swoole_http_response, sendfile)
     }
     if (file_stat.st_size <= offset)
     {
-        swoole_php_error(E_WARNING, "parameter $offset[%ld] exceeds the file size.", offset);
+        swoole_php_error(E_WARNING, "parameter $offset[" ZEND_LONG_FMT "] exceeds the file size.", offset);
         RETURN_FALSE;
     }
     if (length > file_stat.st_size - offset)
     {
-        swoole_php_sys_error(E_WARNING, "parameter $length[%ld] exceeds the file size.", length);
+        swoole_php_sys_error(E_WARNING, "parameter $length[" ZEND_LONG_FMT "] exceeds the file size.", length);
         RETURN_FALSE;
     }
     if (length == 0)
@@ -2088,14 +2090,15 @@ static PHP_METHOD(swoole_http_response, sendfile)
     swString_clear(swoole_http_buffer);
     http_build_header(ctx, getThis(), swoole_http_buffer, length);
 
-    int ret = swServer_tcp_send(SwooleG.serv, ctx->fd, swoole_http_buffer->str, swoole_http_buffer->length);
+    swServer *serv = SwooleG.serv;
+
+    int ret = serv->send(serv, ctx->fd, swoole_http_buffer->str, swoole_http_buffer->length);
     if (ret < 0)
     {
         ctx->send_header = 0;
         RETURN_FALSE;
     }
-
-    ret = swServer_tcp_sendfile(SwooleG.serv, ctx->fd, filename, filename_length, offset, length);
+    ret = serv->sendfile(serv, ctx->fd, filename, filename_length, offset, length);
     if (ret < 0)
     {
         ctx->send_header = 0;
@@ -2103,7 +2106,7 @@ static PHP_METHOD(swoole_http_response, sendfile)
     }
     if (!ctx->keepalive)
     {
-        swServer_tcp_close(SwooleG.serv, ctx->fd, 0);
+        serv->close(serv, ctx->fd, 0);
     }
     swoole_http_context_free(ctx);
     RETURN_TRUE;
@@ -2425,8 +2428,7 @@ static PHP_METHOD(swoole_http_response, redirect)
     //status
     if (http_code)
     {
-        convert_to_long(http_code);
-        ctx->response.status = Z_LVAL_P(http_code);
+        ctx->response.status = zval_get_long(http_code);
     }
     else
     {
