@@ -125,7 +125,7 @@ static int mmap_stream_seek(php_stream *stream, off_t offset, int whence, off_t
             *newoffset = (off_t) -1;
             return -1;
         }
-        res->ptr += offset;
+        res->ptr = res->memory + res->size + offset;
         *newoffset = res->ptr - res->memory;
         return 0;
     default:
@@ -206,7 +206,7 @@ static PHP_METHOD(swoole_mmap, open)
     void *addr = mmap(NULL, size, PROT_WRITE | PROT_READ, MAP_SHARED, fd, offset);
     if (addr == MAP_FAILED)
     {
-        swoole_php_sys_error(E_WARNING, "mmap(%ld) failed.", size);
+        swoole_php_sys_error(E_WARNING, "mmap(" ZEND_LONG_FMT ") failed.", size);
         close(fd);
         RETURN_FALSE;
     }
