@@ -106,12 +106,15 @@ enum mysql_read_state
 
 enum mysql_error_code
 {
+    // FIXME: if it should be bigger than SW_ABORT?
+    // may be in conflict with SW_xxx err code.
     SW_MYSQL_ERR_PROTOCOL_ERROR = 1,
     SW_MYSQL_ERR_BUFFER_OVERSIZE,
     SW_MYSQL_ERR_PACKET_CORRUPT,
     SW_MYSQL_ERR_WANT_READ,
     SW_MYSQL_ERR_WANT_WRITE,
     SW_MYSQL_ERR_UNKNOWN_ERROR,
+
     SW_MYSQL_ERR_MYSQL_ERROR,
     SW_MYSQL_ERR_SERVER_LOST,
     SW_MYSQL_ERR_BAD_PORT,
@@ -339,11 +342,10 @@ typedef struct
     ulong_t num_column;
     ulong_t index_column;
     uint32_t num_row;
-    uint8_t wait_recv;
     uint8_t response_type;
     uint32_t packet_length :24;
     uint32_t packet_number :8;
-    uint32_t error_code;
+    int32_t  error_code;
     uint32_t warnings;
     uint16_t status_code;
     char status_msg[6];
@@ -385,9 +387,13 @@ typedef struct _mysql_client
     zval _object;
     zval _onClose;
 
+    size_t want_length;
     off_t check_offset;
     mysql_response_t response; /* single response */
 
+    // for stored procedure
+    zval* tmp_result;
+
 } mysql_client;
 
 #define mysql_request_buffer (SwooleTG.buffer_stack)
@@ -416,7 +422,10 @@ typedef struct _mysql_client
 #define SW_MYSQL_PACKET_ERR  0xff
 
 /* int<3>	payload_length + int<1>	sequence_id */
-#define SW_MYSQL_PACKET_HEADER_SIZE  4
+#define SW_MYSQL_PACKET_HEADER_SIZE   4
+#define SW_MYSQL_PACKET_EOF_MAX_SIZE  9
+#define SW_MYSQL_MAX_PACKET_BODY_SIZE 0x00ffffff
+#define SW_MYSQL_MAX_PACKET_SIZE      (SW_MYSQL_PACKET_HEADER_SIZE + SW_MYSQL_MAX_PACKET_BODY_SIZE)
 
 #define mysql_uint2korr(A)  (uint16_t) (((uint16_t) ((zend_uchar) (A)[0])) +\
                                ((uint16_t) ((zend_uchar) (A)[1]) << 8))
@@ -515,7 +524,7 @@ static sw_inline void mysql_pack_length(int length, char *buf)
     buf[0] = length;
 }
 
-static sw_inline int mysql_lcb_ll(char *m, ulong_t *r, char *nul, int len)
+static sw_inline int mysql_length_coded_binary(char *m, ulong_t *r, char *nul, uint32_t len)
 {
     if (len < 1)
     {
@@ -588,14 +597,6 @@ static sw_inline int mysql_write_lcb(char *p, long val)
     }
 }
 
-static sw_inline int mysql_length_coded_binary(char *m, ulong_t *r, char *nul, int len)
-{
-    ulong_t val = 0;
-    int retcode = mysql_lcb_ll(m, &val, nul, len);
-    *r = val;
-    return retcode;
-}
-
 int mysql_query(zval *zobject, mysql_client *client, swString *sql, zval *callback);
 
 END_EXTERN_C()
