@@ -16,9 +16,8 @@
  +----------------------------------------------------------------------+
  */
 
-#include "php_swoole.h"
+#include "php_swoole_cxx.h"
 
-#ifdef SW_COROUTINE
 #include "swoole_coroutine.h"
 #include "swoole_mysql.h"
 
@@ -230,7 +229,6 @@ static int swoole_mysql_coro_execute(zval *zobject, mysql_client *client, zval *
     long lval;
     char buf[10];
     zval *value;
-    zval _value;
 
     uint16_t param_count = 0;
     if (params)
@@ -297,39 +295,33 @@ static int swoole_mysql_coro_execute(zval *zobject, mysql_client *client, zval *
             else
             {
                 mysql_int2store((mysql_request_buffer->str + type_start_offset) + (index * 2), SW_MYSQL_TYPE_VAR_STRING);
-                ZVAL_DUP(&_value, value);
-                value = &_value;
-                convert_to_string(value);
-                if (Z_STRLEN_P(value) > 0xffff)
+                zend::string str_value(value);
+
+                if (str_value.len() > 0xffff)
                 {
                     buf[0] = (char) SW_MYSQL_TYPE_VAR_STRING;
                     if (swString_append_ptr(mysql_request_buffer, buf, 1) < 0)
                     {
-                        zval_dtor(value);
                         return SW_ERR;
                     }
                 }
-                else if (Z_STRLEN_P(value) > 250)
+                else if (str_value.len() > 250)
                 {
                     buf[0] = (char) SW_MYSQL_TYPE_BLOB;
                     if (swString_append_ptr(mysql_request_buffer, buf, 1) < 0)
                     {
-                        zval_dtor(value);
                         return SW_ERR;
                     }
                 }
-                lval = mysql_write_lcb(buf, Z_STRLEN_P(value));
+                lval = mysql_write_lcb(buf, str_value.len());
                 if (swString_append_ptr(mysql_request_buffer, buf, lval) < 0)
                 {
-                    zval_dtor(value);
                     return SW_ERR;
                 }
-                if (swString_append_ptr(mysql_request_buffer, Z_STRVAL_P(value), Z_STRLEN_P(value)) < 0)
+                if (swString_append_ptr(mysql_request_buffer, str_value.val(), str_value.len()) < 0)
                 {
-                    zval_dtor(value);
                     return SW_ERR;
                 }
-                zval_dtor(value);
             }
             index++;
        }
@@ -620,13 +612,34 @@ static int swoole_mysql_coro_close(zval *zobject)
         client->statement_list = NULL;
     }
 
+    //clear connector
+    if (client->connector.host)
+    {
+        efree(client->connector.host);
+        client->connector.host = NULL;
+    }
+    if (client->connector.user)
+    {
+        efree(client->connector.user);
+        client->connector.user = NULL;
+    }
+    if (client->connector.password)
+    {
+        efree(client->connector.password);
+        client->connector.password = NULL;
+    }
+    if (client->connector.database)
+    {
+        efree(client->connector.database);
+        client->connector.database = NULL;
+    }
+
     client->cli->close(client->cli);
     swClient_free(client->cli);
     efree(client->cli);
     client->cli = NULL;
     client->state = SW_MYSQL_STATE_CLOSED;
     client->iowait = SW_MYSQL_CORO_STATUS_CLOSED;
-    //TODO: clear connector
 
     return SUCCESS;
 }
@@ -660,12 +673,16 @@ static PHP_METHOD(swoole_mysql_coro, connect)
     }
 
     mysql_connector *connector = &client->connector;
+    zend::string str_host;
+    zend::string str_user;
+    zend::string str_database;
+    zend::string str_password;
 
     if (php_swoole_array_get_value(_ht, "host", value))
     {
-        convert_to_string(value);
-        connector->host = Z_STRVAL_P(value);
-        connector->host_len = Z_STRLEN_P(value);
+        str_host = value;
+        connector->host = str_host.val();
+        connector->host_len = str_host.len();
     }
     else
     {
@@ -675,8 +692,7 @@ static PHP_METHOD(swoole_mysql_coro, connect)
     }
     if (php_swoole_array_get_value(_ht, "port", value))
     {
-        convert_to_long(value);
-        connector->port = Z_LVAL_P(value);
+        connector->port = zval_get_long(value);
     }
     else
     {
@@ -684,9 +700,9 @@ static PHP_METHOD(swoole_mysql_coro, connect)
     }
     if (php_swoole_array_get_value(_ht, "user", value))
     {
-        convert_to_string(value);
-        connector->user = Z_STRVAL_P(value);
-        connector->user_len = Z_STRLEN_P(value);
+        str_user = value;
+        connector->user = str_user.val();
+        connector->user_len = str_user.len();
     }
     else
     {
@@ -696,9 +712,9 @@ static PHP_METHOD(swoole_mysql_coro, connect)
     }
     if (php_swoole_array_get_value(_ht, "password", value))
     {
-        convert_to_string(value);
-        connector->password = Z_STRVAL_P(value);
-        connector->password_len = Z_STRLEN_P(value);
+        str_password = value;
+        connector->password = str_password.val();
+        connector->password_len = str_password.len();
     }
     else
     {
@@ -708,9 +724,9 @@ static PHP_METHOD(swoole_mysql_coro, connect)
     }
     if (php_swoole_array_get_value(_ht, "database", value))
     {
-        convert_to_string(value);
-        connector->database = Z_STRVAL_P(value);
-        connector->database_len = Z_STRLEN_P(value);
+        str_database = value;
+        connector->database = str_database.val();
+        connector->database_len = str_database.len();
     }
     else
     {
@@ -720,8 +736,7 @@ static PHP_METHOD(swoole_mysql_coro, connect)
     }
     if (php_swoole_array_get_value(_ht, "timeout", value))
     {
-        convert_to_double(value);
-        connector->timeout = Z_DVAL_P(value);
+        connector->timeout = zval_get_double(value);
     }
     else
     {
@@ -729,12 +744,12 @@ static PHP_METHOD(swoole_mysql_coro, connect)
     }
     if (php_swoole_array_get_value(_ht, "charset", value))
     {
-        convert_to_string(value);
-        connector->character_set = mysql_get_charset(Z_STRVAL_P(value));
+        zend::string str_charset(value);
+        connector->character_set = mysql_get_charset(str_charset.val());
         if (connector->character_set < 0)
         {
             char buf[64];
-            snprintf(buf, sizeof(buf), "unknown charset [%s].", Z_STRVAL_P(value));
+            snprintf(buf, sizeof(buf), "unknown charset [%s].", str_charset.val());
             zend_throw_exception(swoole_mysql_coro_exception_ce_ptr, buf, 11);
             zval_ptr_dtor(server_info);
             RETURN_FALSE;
@@ -747,14 +762,12 @@ static PHP_METHOD(swoole_mysql_coro, connect)
 
     if (php_swoole_array_get_value(_ht, "strict_type", value))
     {
-        convert_to_boolean(value);
-        connector->strict_type = Z_BVAL_P(value);
+        connector->strict_type = zval_is_true(value);
     }
 
     if (php_swoole_array_get_value(_ht, "fetch_mode", value))
     {
-        convert_to_boolean(value);
-        connector->fetch_mode = Z_BVAL_P(value);
+        connector->fetch_mode = zval_is_true(value);
     }
 
     swClient *cli = (swClient *) emalloc(sizeof(swClient));
@@ -844,6 +857,11 @@ static PHP_METHOD(swoole_mysql_coro, connect)
     context->state = SW_CORO_CONTEXT_RUNNING;
     context->coro_params = *getThis();
 
+    connector->host = estrndup(connector->host, connector->host_len);
+    connector->user = estrndup(connector->user, connector->user_len);
+    connector->password = estrndup(connector->password, connector->password_len);
+    connector->database = estrndup(connector->database, connector->database_len);
+
     if (connector->timeout > 0)
     {
         connector->timer = swTimer_add(&SwooleG.timer, (long) (connector->timeout * 1000), 0, context, swoole_mysql_coro_onTimeout);
@@ -1233,13 +1251,12 @@ static PHP_METHOD(swoole_mysql_coro_statement, fetch)
         ZVAL_NEW_REF(stmt->result, stmt->result);
         args[0] = *stmt->result;
 
-        zval *fcn;
-        SW_MAKE_STD_ZVAL(fcn);
-        ZVAL_STRING(fcn, "array_shift");
+        zval fcn;
+        ZVAL_STRING(&fcn, "array_shift");
         int ret;
         zval retval;
-        ret = call_user_function_ex(EG(function_table), NULL, fcn, &retval, 1, args, 0, NULL);
-        zval_ptr_dtor(fcn);
+        ret = call_user_function_ex(EG(function_table), NULL, &fcn, &retval, 1, args, 0, NULL);
+        zval_ptr_dtor(&fcn);
         ZVAL_UNREF(stmt->result);
 
         if (ret == FAILURE)
@@ -1357,12 +1374,6 @@ static PHP_METHOD(swoole_mysql_coro, escape)
         RETURN_FALSE;
     }
 
-    if (str.length <= 0)
-    {
-        swoole_php_fatal_error(E_WARNING, "String is empty.");
-        RETURN_FALSE;
-    }
-
     mysql_client *client = (mysql_client *) swoole_get_object(getThis());
     if (!client)
     {
@@ -1477,7 +1488,7 @@ static void swoole_mysql_coro_onConnect(mysql_client *client)
     zval *zobject = client->object;
 
     zval *retval = NULL;
-    zval *result;
+    zval result;
 
     if (client->connector.timer)
     {
@@ -1485,8 +1496,6 @@ static void swoole_mysql_coro_onConnect(mysql_client *client)
         client->connector.timer = NULL;
     }
 
-    SW_MAKE_STD_ZVAL(result);
-
     //SwooleG.main_reactor->del(SwooleG.main_reactor, client->fd);
 
     if (client->connector.error_code > 0)
@@ -1494,7 +1503,7 @@ static void swoole_mysql_coro_onConnect(mysql_client *client)
         zend_update_property_stringl(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("connect_error"), client->connector.error_msg, client->connector.error_length);
         zend_update_property_long(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("connect_errno"), client->connector.error_code);
 
-        ZVAL_BOOL(result, 0);
+        ZVAL_BOOL(&result, 0);
 
         swoole_mysql_coro_close(zobject);
     }
@@ -1504,14 +1513,13 @@ static void swoole_mysql_coro_onConnect(mysql_client *client)
         client->iowait = SW_MYSQL_CORO_STATUS_READY;
         zend_update_property_bool(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("connected"), 1);
         client->connected = 1;
-        ZVAL_BOOL(result, 1);
+        ZVAL_BOOL(&result, 1);
     }
 
     client->cid = 0;
 
     php_coro_context *sw_current_context = (php_coro_context *) swoole_get_property(zobject, 0);
-    int ret = PHPCoroutine::resume_m(sw_current_context, result, retval);
-    zval_ptr_dtor(result);
+    int ret = PHPCoroutine::resume_m(sw_current_context, &result, retval);
     if (ret == SW_CORO_ERR_END && retval)
     {
         zval_ptr_dtor(retval);
@@ -1847,6 +1855,7 @@ static int swoole_mysql_coro_onRead(swReactor *reactor, swEvent *event)
             }
 
             _active_close:
+            client->state = SW_MYSQL_STATE_QUERY;
             swoole_mysql_coro_close(zobject);
 
             if (!client->cid)
@@ -1872,7 +1881,6 @@ static int swoole_mysql_coro_onRead(swReactor *reactor, swEvent *event)
             {
                 zval_ptr_dtor(retval);
             }
-            client->state = SW_MYSQL_STATE_QUERY;
             return SW_OK;
         }
         else
@@ -1952,5 +1960,3 @@ static int swoole_mysql_coro_onRead(swReactor *reactor, swEvent *event)
     }
     return SW_OK;
 }
-
-#endif
