@@ -75,6 +75,7 @@ ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_process_alarm, 0, 0, 1)
     ZEND_ARG_INFO(0, usec)
+    ZEND_ARG_INFO(0, type)
 ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_process_kill, 0, 0, 1)
@@ -629,11 +630,7 @@ static void php_swoole_onSignal(int signo)
     zval args[1];
     zval *callback = signal_callback[signo];
 
-    zval *zsigno;
-    SW_MAKE_STD_ZVAL(zsigno);
-    ZVAL_LONG(zsigno, signo);
-
-    args[0] = *zsigno;
+    ZVAL_LONG(& args[0], signo);
 
     if (sw_call_user_function_ex(EG(function_table), NULL, callback, &retval, 1, args, 0, NULL) == FAILURE)
     {
@@ -647,7 +644,6 @@ static void php_swoole_onSignal(int signo)
     {
         zval_ptr_dtor(retval);
     }
-    zval_ptr_dtor(zsigno);
 }
 
 zend_bool php_swoole_signal_isset_handler(int signo)
@@ -1071,8 +1067,7 @@ static PHP_METHOD(swoole_process, setaffinity)
     CPU_ZERO(&cpu_set);
 
     SW_HASHTABLE_FOREACH_START(Z_ARRVAL_P(array), value)
-        convert_to_long(value);
-        if (Z_LVAL_P(value) >= SW_CPU_NUM)
+        if (zval_get_long(value) >= SW_CPU_NUM)
         {
             swoole_php_fatal_error(E_WARNING, "invalid cpu id [%d]", (int) Z_LVAL_P(value));
             RETURN_FALSE;
