@@ -69,6 +69,7 @@ static php_stream_ops socket_ops
 typedef struct
 {
     php_netstream_data_t stream;
+    double read_timeout;
     Socket *socket;
 } php_swoole_netstream_data_t;
 
@@ -265,15 +266,16 @@ static size_t socket_read(php_stream *stream, char *buf, size_t count)
     php_swoole_netstream_data_t *abstract = (php_swoole_netstream_data_t *) stream->abstract;
     Socket *sock = (Socket*) abstract->socket;
     ssize_t nr_bytes = 0;
-
     if (!sock)
     {
         return 0;
     }
-
+    sock->set_timeout(abstract->read_timeout);
     nr_bytes = sock->recv(buf, count);
-    stream->eof = (nr_bytes == 0 || nr_bytes == -1);
-
+    /**
+     * sock->errCode != ETIMEDOUT : Compatible with sync blocking IO
+     */
+    stream->eof = (nr_bytes == 0 || (nr_bytes == -1 && sock->errCode != ETIMEDOUT && swConnection_error(sock->errCode) == SW_CLOSE));
     if (nr_bytes > 0)
     {
         php_stream_notify_progress_increment(PHP_STREAM_CONTEXT(stream), nr_bytes, 0);
@@ -295,8 +297,13 @@ static int socket_flush(php_stream *stream)
 static int socket_close(php_stream *stream, int close_handle)
 {
     php_swoole_netstream_data_t *abstract = (php_swoole_netstream_data_t *) stream->abstract;
+    if (!abstract)
+    {
+        return 0;
+    }
+    stream->abstract = NULL;
     Socket *sock = (Socket*) abstract->socket;
-    delete sock;
+    sock->close();
     efree(abstract);
     return 0;
 }
@@ -453,7 +460,7 @@ static inline int socket_accept(php_stream *stream, Socket *sock, php_stream_xpo
 
     if ((NULL != PHP_STREAM_CONTEXT(stream))
             && (tmpzval = php_stream_context_get_option(PHP_STREAM_CONTEXT(stream), "socket", "tcp_nodelay")) != NULL
-            && zend_is_true(tmpzval))
+            && zval_is_true(tmpzval))
     {
         tcp_nodelay = 1;
     }
@@ -543,7 +550,6 @@ static inline int socket_recvfrom(Socket *sock, char *buf, size_t buflen, zend_s
         php_sockaddr_storage sa;
         socklen_t sl = sizeof(sa);
         ret = sock->recvfrom(buf, buflen, (struct sockaddr*) &sa, &sl);
-        ret = (ret == SOCK_CONN_ERR) ? -1 : ret;
         if (sl)
         {
             php_network_populate_name_from_sockaddr((struct sockaddr*) &sa, sl, textaddr, addr, addrlen);
@@ -564,7 +570,6 @@ static inline int socket_recvfrom(Socket *sock, char *buf, size_t buflen, zend_s
     else
     {
         ret = sock->recv(buf, buflen);
-        ret = (ret == SOCK_CONN_ERR) ? -1 : ret;
     }
 
     return ret;
@@ -652,7 +657,7 @@ static inline int socket_xport_api(php_stream *stream, Socket *sock, php_stream_
 
 #ifdef SO_REUSEPORT
             if ((tmpzval = php_stream_context_get_option(ctx, "socket", "so_reuseport")) != NULL
-                    && zend_is_true(tmpzval))
+                    && zval_is_true(tmpzval))
             {
                 setsockopt(sock->get_fd(), SOL_SOCKET, SO_REUSEPORT, (char*) &sockoptval, sizeof(sockoptval));
             }
@@ -660,7 +665,7 @@ static inline int socket_xport_api(php_stream *stream, Socket *sock, php_stream_
 
 #ifdef SO_BROADCAST
             if ((tmpzval = php_stream_context_get_option(ctx, "socket", "so_broadcast")) != NULL
-                    && zend_is_true(tmpzval))
+                    && zval_is_true(tmpzval))
             {
                 setsockopt(sock->get_fd(), SOL_SOCKET, SO_BROADCAST, (char*) &sockoptval, sizeof(sockoptval));
             }
@@ -735,6 +740,7 @@ static int socket_set_option(php_stream *stream, int option, int value, void *pt
 {
     php_swoole_netstream_data_t *abstract = (php_swoole_netstream_data_t *) stream->abstract;
     Socket *sock = (Socket*) abstract->socket;
+    struct timeval default_timeout = { 0, 0 };
     switch (option)
     {
     case PHP_STREAM_OPTION_XPORT_API:
@@ -783,11 +789,14 @@ static int socket_set_option(php_stream *stream, int option, int value, void *pt
             add_assoc_zval((zval *)ptrparam, "crypto", &tmp);
         }
 #endif
+        add_assoc_bool((zval * )ptrparam, "timed_out", sock->errCode == ETIMEDOUT);
         add_assoc_bool((zval * )ptrparam, "eof", stream->eof);
+        add_assoc_bool((zval * )ptrparam, "blocked", 1);
         break;
 
     case PHP_STREAM_OPTION_READ_TIMEOUT:
-        sock->set_timeout((struct timeval*) ptrparam);
+        default_timeout = *(struct timeval*) ptrparam;
+        abstract->read_timeout = (double) default_timeout.tv_sec + ((double) default_timeout.tv_usec / 1000 / 1000);
         break;
 #ifdef SW_USE_OPENSSL
     case PHP_STREAM_OPTION_CRYPTO_API:
@@ -910,7 +919,9 @@ static php_stream *socket_create(
     abstract->socket = sock;
     abstract->stream.timeout.tv_sec = FG(default_socket_timeout);
     abstract->stream.socket = sock->get_fd();
+    abstract->read_timeout = (double)FG(default_socket_timeout);
 
+    persistent_id = nullptr;//prevent stream api in user level using pconnect to persist the socket
     stream = php_stream_alloc_rel(&socket_ops, abstract, persistent_id, "r+");
 
     if (stream == NULL)
