@@ -14,14 +14,15 @@
   +----------------------------------------------------------------------+
  */
 
-#include "php_swoole.h"
+#include "php_swoole_cxx.h"
 
 #ifdef SW_COROUTINE
 #include "swoole_coroutine.h"
 #endif
 
-zend_class_entry swoole_server_port_ce;
-zend_class_entry *swoole_server_port_class_entry_ptr;
+static zend_class_entry swoole_server_port_ce;
+zend_class_entry *swoole_server_port_ce_ptr;
+static zend_object_handlers swoole_server_port_handlers;
 
 static PHP_METHOD(swoole_server_port, __construct);
 static PHP_METHOD(swoole_server_port, __destruct);
@@ -58,35 +59,29 @@ const zend_function_entry swoole_server_port_methods[] =
 
 void swoole_server_port_init(int module_number)
 {
-    SWOOLE_INIT_CLASS_ENTRY(swoole_server_port_ce, "swoole_server_port", "Swoole\\Server\\Port", swoole_server_port_methods);
-    swoole_server_port_class_entry_ptr = zend_register_internal_class(&swoole_server_port_ce);
-    swoole_server_port_class_entry_ptr->serialize = zend_class_serialize_deny;
-    swoole_server_port_class_entry_ptr->unserialize = zend_class_unserialize_deny;
-    SWOOLE_CLASS_ALIAS(swoole_server_port, "Swoole\\Server\\Port");
+    SWOOLE_INIT_CLASS_ENTRY(swoole_server_port, "Swoole\\Server\\Port", "swoole_server_port", NULL, swoole_server_port_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_server_port, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_server_port, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_server_port, zend_class_unset_property_deny);
 
-    if (SWOOLE_G(use_shortname))
-    {
-        sw_zend_register_class_alias("Co\\Server\\Port", swoole_server_port_class_entry_ptr);
-    }
+    zend_declare_property_null(swoole_server_port_ce_ptr, ZEND_STRL("onConnect"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_port_ce_ptr, ZEND_STRL("onReceive"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_port_ce_ptr, ZEND_STRL("onClose"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_port_ce_ptr, ZEND_STRL("onPacket"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_port_ce_ptr, ZEND_STRL("onBufferFull"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_port_ce_ptr, ZEND_STRL("onBufferEmpty"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_port_ce_ptr, ZEND_STRL("onRequest"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_port_ce_ptr, ZEND_STRL("onHandShake"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_port_ce_ptr, ZEND_STRL("onMessage"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_port_ce_ptr, ZEND_STRL("onOpen"), ZEND_ACC_PRIVATE);
 
-    zend_declare_property_null(swoole_server_port_class_entry_ptr, ZEND_STRL("onConnect"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_port_class_entry_ptr, ZEND_STRL("onReceive"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_port_class_entry_ptr, ZEND_STRL("onClose"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_port_class_entry_ptr, ZEND_STRL("onPacket"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_port_class_entry_ptr, ZEND_STRL("onBufferFull"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_port_class_entry_ptr, ZEND_STRL("onBufferEmpty"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_port_class_entry_ptr, ZEND_STRL("onRequest"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_port_class_entry_ptr, ZEND_STRL("onHandShake"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_port_class_entry_ptr, ZEND_STRL("onMessage"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_port_class_entry_ptr, ZEND_STRL("onOpen"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_server_port_ce_ptr, ZEND_STRL("host"), ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_server_port_ce_ptr, ZEND_STRL("port"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_server_port_ce_ptr, ZEND_STRL("type"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_server_port_ce_ptr, ZEND_STRL("sock"), -1, ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_server_port_ce_ptr, ZEND_STRL("setting"), ZEND_ACC_PUBLIC);
 
-    zend_declare_property_null(swoole_server_port_class_entry_ptr, ZEND_STRL("host"), ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_server_port_class_entry_ptr, ZEND_STRL("port"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_server_port_class_entry_ptr, ZEND_STRL("type"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_server_port_class_entry_ptr, ZEND_STRL("sock"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_port_class_entry_ptr, ZEND_STRL("setting"), ZEND_ACC_PUBLIC);
-
-    zend_declare_property_null(swoole_server_port_class_entry_ptr, ZEND_STRL("connections"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_server_port_ce_ptr, ZEND_STRL("connections"), ZEND_ACC_PUBLIC);
 }
 
 static PHP_METHOD(swoole_server_port, __construct)
@@ -143,16 +138,14 @@ static PHP_METHOD(swoole_server_port, set)
     //backlog
     if (php_swoole_array_get_value(vht, "backlog", v))
     {
-        convert_to_long(v);
-        port->backlog = (int) Z_LVAL_P(v);
+        port->backlog = (int) zval_get_long(v);
     }
     if (php_swoole_array_get_value(vht, "socket_buffer_size", v))
     {
-        convert_to_long(v);
-        port->socket_buffer_size = (int) Z_LVAL_P(v);
+        port->socket_buffer_size = (int) zval_get_long(v);
         if (port->socket_buffer_size <= 0)
         {
-            port->socket_buffer_size = SW_MAX_INT;
+            port->socket_buffer_size = INT_MAX;
         }
     }
     /**
@@ -160,11 +153,10 @@ static PHP_METHOD(swoole_server_port, set)
      */
     if (php_swoole_array_get_value(vht, "kernel_socket_recv_buffer_size", v))
     {
-        convert_to_long(v);
-        port->kernel_socket_recv_buffer_size = (int) Z_LVAL_P(v);
+        port->kernel_socket_recv_buffer_size = (int) zval_get_long(v);
         if (port->kernel_socket_recv_buffer_size <= 0)
         {
-            port->kernel_socket_recv_buffer_size = SW_MAX_INT;
+            port->kernel_socket_recv_buffer_size = INT_MAX;
         }
     }
     /**
@@ -172,28 +164,24 @@ static PHP_METHOD(swoole_server_port, set)
      */
     if (php_swoole_array_get_value(vht, "kernel_socket_send_buffer_size", v))
     {
-        convert_to_long(v);
-        port->kernel_socket_send_buffer_size = (int) Z_LVAL_P(v);
+        port->kernel_socket_send_buffer_size = (int) zval_get_long(v);
         if (port->kernel_socket_send_buffer_size <= 0)
         {
-            port->kernel_socket_send_buffer_size = SW_MAX_INT;
+            port->kernel_socket_send_buffer_size = INT_MAX;
         }
     }
     if (php_swoole_array_get_value(vht, "buffer_high_watermark", v))
     {
-        convert_to_long(v);
-        port->buffer_high_watermark = (int) Z_LVAL_P(v);
+        port->buffer_high_watermark = (int) zval_get_long(v);
     }
     if (php_swoole_array_get_value(vht, "buffer_low_watermark", v))
     {
-        convert_to_long(v);
-        port->buffer_low_watermark = (int) Z_LVAL_P(v);
+        port->buffer_low_watermark = (int) zval_get_long(v);
     }
     //server: tcp_nodelay
     if (php_swoole_array_get_value(vht, "open_tcp_nodelay", v))
     {
-        convert_to_boolean(v);
-        port->open_tcp_nodelay = Z_BVAL_P(v);
+        port->open_tcp_nodelay = zval_is_true(v);
     }
     else
     {
@@ -202,26 +190,22 @@ static PHP_METHOD(swoole_server_port, set)
     //tcp_defer_accept
     if (php_swoole_array_get_value(vht, "tcp_defer_accept", v))
     {
-        convert_to_long(v);
-        port->tcp_defer_accept = (uint8_t) Z_LVAL_P(v);
+        port->tcp_defer_accept = (uint8_t) zval_get_long(v);
     }
     //tcp_keepalive
     if (php_swoole_array_get_value(vht, "open_tcp_keepalive", v))
     {
-        convert_to_boolean(v);
-        port->open_tcp_keepalive = Z_BVAL_P(v);
+        port->open_tcp_keepalive = zval_is_true(v);
     }
     //buffer: eof check
     if (php_swoole_array_get_value(vht, "open_eof_check", v))
     {
-        convert_to_boolean(v);
-        port->open_eof_check = Z_BVAL_P(v);
+        port->open_eof_check = zval_is_true(v);
     }
     //buffer: split package with eof
     if (php_swoole_array_get_value(vht, "open_eof_split", v))
     {
-        convert_to_boolean(v);
-        port->protocol.split_by_eof = Z_BVAL_P(v);
+        port->protocol.split_by_eof = zval_is_true(v);
         if (port->protocol.split_by_eof)
         {
             port->open_eof_check = 1;
@@ -230,8 +214,8 @@ static PHP_METHOD(swoole_server_port, set)
     //package eof
     if (php_swoole_array_get_value(vht, "package_eof", v))
     {
-        convert_to_string(v);
-        port->protocol.package_eof_len = Z_STRLEN_P(v);
+        zend::string str_v(v);
+        port->protocol.package_eof_len = str_v.len();
         if (port->protocol.package_eof_len == 0)
         {
             swoole_php_fatal_error(E_ERROR, "pacakge_eof cannot be an empty string");
@@ -243,19 +227,17 @@ static PHP_METHOD(swoole_server_port, set)
             RETURN_FALSE;
         }
         bzero(port->protocol.package_eof, SW_DATA_EOF_MAXLEN);
-        memcpy(port->protocol.package_eof, Z_STRVAL_P(v), Z_STRLEN_P(v));
+        memcpy(port->protocol.package_eof, str_v.val(), str_v.len());
     }
     //http_protocol
     if (php_swoole_array_get_value(vht, "open_http_protocol", v))
     {
-        convert_to_boolean(v);
-        port->open_http_protocol = Z_BVAL_P(v);
+        port->open_http_protocol = zval_is_true(v);
     }
     //websocket protocol
     if (php_swoole_array_get_value(vht, "open_websocket_protocol", v))
     {
-        convert_to_boolean(v);
-        port->open_websocket_protocol = Z_BVAL_P(v);
+        port->open_websocket_protocol = zval_is_true(v);
         if (port->open_websocket_protocol)
         {
             port->open_http_protocol = 1;
@@ -264,74 +246,65 @@ static PHP_METHOD(swoole_server_port, set)
     }
     if (php_swoole_array_get_value(vht, "websocket_subprotocol", v))
     {
-        convert_to_string(v);
+        zend::string str_v(v);
         if (port->websocket_subprotocol)
         {
             sw_free(port->websocket_subprotocol);
         }
-        port->websocket_subprotocol = sw_strdup(Z_STRVAL_P(v));
-        port->websocket_subprotocol_length = Z_STRLEN_P(v);
+        port->websocket_subprotocol = sw_strndup(str_v.val(), str_v.len());
+        port->websocket_subprotocol_length = str_v.len();
     }
     if (php_swoole_array_get_value(vht, "open_websocket_close_frame", v))
     {
-        convert_to_boolean(v);
-        port->open_websocket_close_frame = Z_BVAL_P(v);
+        port->open_websocket_close_frame = zval_is_true(v);
     }
 #ifdef SW_USE_HTTP2
     //http2 protocol
     if (php_swoole_array_get_value(vht, "open_http2_protocol", v))
     {
-        convert_to_boolean(v);
-        port->open_http2_protocol = Z_BVAL_P(v);
+        port->open_http2_protocol = zval_is_true(v);
     }
 #endif
     //buffer: mqtt protocol
     if (php_swoole_array_get_value(vht, "open_mqtt_protocol", v))
     {
-        convert_to_boolean(v);
-        port->open_mqtt_protocol = Z_BVAL_P(v);
+        port->open_mqtt_protocol = zval_is_true(v);
     }
     //redis protocol
     if (php_swoole_array_get_value(vht, "open_redis_protocol", v))
     {
-        convert_to_boolean(v);
-        port->open_redis_protocol = Z_BVAL_P(v);
+        port->open_redis_protocol = zval_get_long(v);
     }
     //tcp_keepidle
     if (php_swoole_array_get_value(vht, "tcp_keepidle", v))
     {
-        convert_to_long(v);
-        port->tcp_keepidle = (uint16_t) Z_LVAL_P(v);
+        port->tcp_keepidle = (uint16_t) zval_get_long(v);
     }
     //tcp_keepinterval
     if (php_swoole_array_get_value(vht, "tcp_keepinterval", v))
     {
-        convert_to_long(v);
-        port->tcp_keepinterval = (uint16_t) Z_LVAL_P(v);
+        port->tcp_keepinterval = (uint16_t) zval_get_long(v);
     }
     //tcp_keepcount
-    if ((v = zend_hash_str_find(vht, ZEND_STRL("tcp_keepcount"))))
+    if (php_swoole_array_get_value(vht, "tcp_keepcount", v))
     {
-        convert_to_long(v);
-        port->tcp_keepcount = (uint16_t) Z_LVAL_P(v);
+        port->tcp_keepcount = (uint16_t) zval_get_long(v);
     }
     //tcp_fastopen
-    if ((v = zend_hash_str_find(vht, ZEND_STRL("tcp_fastopen"))))
+    if (php_swoole_array_get_value(vht, "tcp_fastopen", v))
     {
-        convert_to_boolean(v);
-        port->tcp_fastopen = Z_BVAL_P(v);
+        port->tcp_fastopen = zval_is_true(v);
     }
     //open length check
     if (php_swoole_array_get_value(vht, "open_length_check", v))
     {
-        convert_to_boolean(v);
-        port->open_length_check = Z_BVAL_P(v);
+        port->open_length_check = zval_is_true(v);
     }
     //package length size
     if (php_swoole_array_get_value(vht, "package_length_type", v))
     {
-        convert_to_string(v);
-        port->protocol.package_length_type = Z_STRVAL_P(v)[0];
+        zend::string str_v(v);
+        port->protocol.package_length_type = str_v.val()[0];
         port->protocol.package_length_size = swoole_type_size(port->protocol.package_length_type);
         if (port->protocol.package_length_size == 0)
         {
@@ -339,6 +312,24 @@ static PHP_METHOD(swoole_server_port, set)
             RETURN_FALSE;
         }
     }
+    //package length offset
+    if (php_swoole_array_get_value(vht, "package_length_offset", v))
+    {
+        port->protocol.package_length_offset = (int) zval_get_long(v);
+        if (port->protocol.package_length_offset > SW_IPC_BUFFER_SIZE)
+        {
+            swoole_php_fatal_error(E_ERROR, "'package_length_offset' value is too large.");
+        }
+    }
+    //package body start
+    if (php_swoole_array_get_value(vht, "package_body_offset", v) || php_swoole_array_get_value(vht, "package_body_start", v))
+    {
+        port->protocol.package_body_offset = (int) zval_get_long(v);
+        if (port->protocol.package_body_offset > SW_IPC_BUFFER_SIZE)
+        {
+            swoole_php_fatal_error(E_ERROR, "'package_body_offset' value is too large.");
+        }
+    }
     //length function
     if (php_swoole_array_get_value(vht, "package_length_func", v))
     {
@@ -358,11 +349,15 @@ static PHP_METHOD(swoole_server_port, set)
             if (!sw_zend_is_callable(v, 0, &func_name))
             {
                 swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
-                efree(func_name);
                 return;
             }
             efree(func_name);
             port->protocol.get_package_length = php_swoole_length_func;
+            if (port->protocol.private_data)
+            {
+                zval_ptr_dtor((zval *)port->protocol.private_data);
+                efree(port->protocol.private_data);
+            }
             Z_TRY_ADDREF_P(v);
             port->protocol.private_data = sw_zval_dup(v);
             break;
@@ -370,35 +365,14 @@ static PHP_METHOD(swoole_server_port, set)
 
         port->protocol.package_length_size = 0;
         port->protocol.package_length_type = '\0';
-        port->protocol.package_length_offset = SW_BUFFER_SIZE;
-    }
-    //package length offset
-    if (php_swoole_array_get_value(vht, "package_length_offset", v))
-    {
-        convert_to_long(v);
-        port->protocol.package_length_offset = (int) Z_LVAL_P(v);
-        if (port->protocol.package_length_offset > SW_BUFFER_SIZE)
-        {
-            swoole_php_fatal_error(E_ERROR, "'package_length_offset' value is too large.");
-        }
-    }
-    //package body start
-    if (php_swoole_array_get_value(vht, "package_body_offset", v) || php_swoole_array_get_value(vht, "package_body_start", v))
-    {
-        convert_to_long(v);
-        port->protocol.package_body_offset = (int) Z_LVAL_P(v);
-        if (port->protocol.package_body_offset > SW_BUFFER_SIZE)
-        {
-            swoole_php_fatal_error(E_ERROR, "'package_body_offset' value is too large.");
-        }
+        port->protocol.package_length_offset = SW_IPC_BUFFER_SIZE;
     }
     /**
      * package max length
      */
     if (php_swoole_array_get_value(vht, "package_max_length", v))
     {
-        convert_to_long(v);
-        port->protocol.package_max_length = (int) Z_LVAL_P(v);
+        port->protocol.package_max_length = (int) zval_get_long(v);
     }
 
 #ifdef SW_USE_OPENSSL
@@ -406,109 +380,108 @@ static PHP_METHOD(swoole_server_port, set)
     {
         if (php_swoole_array_get_value(vht, "ssl_cert_file", v))
         {
-            convert_to_string(v);
-            if (access(Z_STRVAL_P(v), R_OK) < 0)
+            zend::string str_v(v);
+            if (access(str_v.val(), R_OK) < 0)
             {
-                swoole_php_fatal_error(E_ERROR, "ssl cert file[%s] not found.", Z_STRVAL_P(v));
+                swoole_php_fatal_error(E_ERROR, "ssl cert file[%s] not found.", str_v.val());
                 return;
             }
             if (port->ssl_option.cert_file)
             {
                 sw_free(port->ssl_option.cert_file);
             }
-            port->ssl_option.cert_file = sw_strdup(Z_STRVAL_P(v));
+            port->ssl_option.cert_file = sw_strndup(str_v.val(), str_v.len());
             port->open_ssl_encrypt = 1;
         }
         if (php_swoole_array_get_value(vht, "ssl_key_file", v))
         {
-            convert_to_string(v);
-            if (access(Z_STRVAL_P(v), R_OK) < 0)
+            zend::string str_v(v);
+            if (access(str_v.val(), R_OK) < 0)
             {
-                swoole_php_fatal_error(E_ERROR, "ssl key file[%s] not found.", Z_STRVAL_P(v));
+                swoole_php_fatal_error(E_ERROR, "ssl key file[%s] not found.", str_v.val());
                 return;
             }
             if (port->ssl_option.key_file)
             {
                 sw_free(port->ssl_option.key_file);
             }
-            port->ssl_option.key_file = sw_strdup(Z_STRVAL_P(v));
+            port->ssl_option.key_file = sw_strndup(str_v.val(), str_v.len());
         }
         if (php_swoole_array_get_value(vht, "ssl_method", v))
         {
-            convert_to_long(v);
-            port->ssl_option.method = (int) Z_LVAL_P(v);
+            port->ssl_option.method = (int) zval_get_long(v);
+        }
+        if (php_swoole_array_get_value(vht, "ssl_verify_peer", v))
+        {
+            port->ssl_option.verify_peer = zval_is_true(v);
+        }
+        if (php_swoole_array_get_value(vht, "ssl_allow_self_signed", v))
+        {
+            port->ssl_option.allow_self_signed = zval_is_true(v);
         }
         //verify client cert
         if (php_swoole_array_get_value(vht, "ssl_client_cert_file", v))
         {
-            convert_to_string(v);
-            if (access(Z_STRVAL_P(v), R_OK) < 0)
+            zend::string str_v(v);
+            if (access(str_v.val(), R_OK) < 0)
             {
-                swoole_php_fatal_error(E_ERROR, "ssl cert file[%s] not found.", port->ssl_option.cert_file);
+                swoole_php_fatal_error(E_ERROR, "ssl_client_cert_file[%s] not found.", str_v.val());
                 return;
             }
             if (port->ssl_option.client_cert_file)
             {
                 sw_free(port->ssl_option.client_cert_file);
             }
-            port->ssl_option.client_cert_file = sw_strdup(Z_STRVAL_P(v));
+            port->ssl_option.client_cert_file = sw_strndup(str_v.val(), str_v.len());
         }
         if (php_swoole_array_get_value(vht, "ssl_verify_depth", v))
         {
-            convert_to_long(v);
-            port->ssl_option.verify_depth = (int) Z_LVAL_P(v);
+            port->ssl_option.verify_depth = (int) zval_get_long(v);
         }
         if (php_swoole_array_get_value(vht, "ssl_prefer_server_ciphers", v))
         {
-            convert_to_boolean(v);
-            port->ssl_config.prefer_server_ciphers = Z_BVAL_P(v);
+            port->ssl_config.prefer_server_ciphers = zval_is_true(v);
         }
         //    if ((v = zend_hash_str_find(vht, ZEND_STRL("ssl_session_tickets"))))
         //    {
-        //        convert_to_boolean(v);
-        //        port->ssl_config.session_tickets = Z_BVAL_P(v);
+        //        port->ssl_config.session_tickets = zval_is_true(v);
         //    }
         //    if ((v = zend_hash_str_find(vht, ZEND_STRL("ssl_stapling"))))
         //    {
-        //        convert_to_boolean(v);
-        //        port->ssl_config.stapling = Z_BVAL_P(v);
+        //        port->ssl_config.stapling = zval_is_true(v);
         //    }
         //    if ((v = zend_hash_str_find(vht, ZEND_STRL("ssl_stapling_verify"))))
         //    {
-        //        convert_to_boolean(v);
-        //        port->ssl_config.stapling_verify = Z_BVAL_P(v);
+        //        port->ssl_config.stapling_verify = zval_is_true(v);
         //    }
         if (php_swoole_array_get_value(vht, "ssl_ciphers", v))
         {
-            convert_to_string(v);
             if (port->ssl_config.ciphers)
             {
                 sw_free(port->ssl_config.ciphers);
             }
-            port->ssl_config.ciphers = sw_strdup(Z_STRVAL_P(v));
+            port->ssl_config.ciphers = zend::string::dup(v);
         }
         if (php_swoole_array_get_value(vht, "ssl_ecdh_curve", v))
         {
-            convert_to_string(v);
+            zend::string str_v(v);
             if (port->ssl_config.ecdh_curve)
             {
                 sw_free(port->ssl_config.ecdh_curve);
             }
-            port->ssl_config.ecdh_curve = sw_strdup(Z_STRVAL_P(v));
+            port->ssl_config.ecdh_curve = zend::string::dup(v);
         }
         if (php_swoole_array_get_value(vht, "ssl_dhparam", v))
         {
-            convert_to_string(v);
             if (port->ssl_config.dhparam)
             {
                 sw_free(port->ssl_config.dhparam);
             }
-            port->ssl_config.dhparam = sw_strdup(Z_STRVAL_P(v));
+            port->ssl_config.dhparam = zend::string::dup(v);
         }
         //    if ((v = zend_hash_str_find(vht, ZEND_STRL("ssl_session_cache"))))
         //    {
-        //        convert_to_string(v);
-        //        port->ssl_config.session_cache = strdup(Z_STRVAL_P(v));
+        //        port->ssl_config.session_cache = zend::string_dup(v);
         //    }
         if (swPort_enable_ssl_encrypt(port) < 0)
         {
@@ -518,7 +491,7 @@ static PHP_METHOD(swoole_server_port, set)
     }
 #endif
 
-    zval *zsetting = sw_zend_read_property_array(swoole_server_port_class_entry_ptr, getThis(), ZEND_STRL("setting"), 1);
+    zval *zsetting = sw_zend_read_property_array(swoole_server_port_ce_ptr, getThis(), ZEND_STRL("setting"), 1);
     php_array_merge(Z_ARRVAL_P(zsetting), Z_ARRVAL_P(zset));
     zval_ptr_dtor(zset);
 }
@@ -543,11 +516,10 @@ static PHP_METHOD(swoole_server_port, on)
     }
 
     char *func_name = NULL;
-    zend_fcall_info_cache *func_cache = (zend_fcall_info_cache *) emalloc(sizeof(zend_fcall_info_cache));
-    if (!sw_zend_is_callable_ex(cb, NULL, 0, &func_name, NULL, func_cache, NULL))
+    zend_fcall_info_cache *fci_cache = (zend_fcall_info_cache *) emalloc(sizeof(zend_fcall_info_cache));
+    if (!sw_zend_is_callable_ex(cb, NULL, 0, &func_name, NULL, fci_cache, NULL))
     {
         swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
-        efree(func_name);
         return;
     }
     efree(func_name);
@@ -579,27 +551,32 @@ static PHP_METHOD(swoole_server_port, on)
         memcpy(property_name + 2, callback_name[i], len);
         l_property_name = len + 2;
         property_name[l_property_name] = '\0';
-        zend_update_property(swoole_server_port_class_entry_ptr, getThis(), property_name, l_property_name, cb);
-        property->callbacks[i] = sw_zend_read_property(swoole_server_port_class_entry_ptr, getThis(), property_name, l_property_name, 0);
+        zend_update_property(swoole_server_port_ce_ptr, getThis(), property_name, l_property_name, cb);
+        property->callbacks[i] = sw_zend_read_property(swoole_server_port_ce_ptr, getThis(), property_name, l_property_name, 0);
         sw_copy_to_stack(property->callbacks[i], property->_callbacks[i]);
+        if (property->caches[i])
+        {
+            efree(property->caches[i]);
+        }
+        property->caches[i] = fci_cache;
 
-        if (i == SW_SERVER_CB_onConnect && serv->onConnect == NULL)
+        if (i == SW_SERVER_CB_onConnect && !serv->onConnect)
         {
             serv->onConnect = php_swoole_onConnect;
         }
-        else if (i == SW_SERVER_CB_onPacket && serv->onPacket == NULL)
+        else if (i == SW_SERVER_CB_onPacket && !serv->onPacket)
         {
             serv->onPacket = php_swoole_onPacket;
         }
-        else if (i == SW_SERVER_CB_onClose && serv->onClose == NULL)
+        else if (i == SW_SERVER_CB_onClose && !serv->onClose)
         {
             serv->onClose = php_swoole_onClose;
         }
-        else if (i == SW_SERVER_CB_onBufferFull && serv->onBufferFull == NULL)
+        else if (i == SW_SERVER_CB_onBufferFull && !serv->onBufferFull)
         {
             serv->onBufferFull = php_swoole_onBufferFull;
         }
-        else if (i == SW_SERVER_CB_onBufferEmpty && serv->onBufferEmpty == NULL)
+        else if (i == SW_SERVER_CB_onBufferEmpty && !serv->onBufferEmpty)
         {
             serv->onBufferEmpty = php_swoole_onBufferEmpty;
         }
@@ -607,14 +584,13 @@ static PHP_METHOD(swoole_server_port, on)
         {
             serv->onReceive = php_swoole_http_onReceive;
         }
-        property->caches[i] = func_cache;
         break;
     }
 
     if (l_property_name == 0)
     {
         swoole_php_error(E_WARNING, "unknown event types[%s]", name);
-        efree(func_cache);
+        efree(fci_cache);
         RETURN_FALSE;
     }
     RETURN_TRUE;
