@@ -50,7 +50,7 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_table_get, 0, 0, 1)
     ZEND_ARG_INFO(0, field)
 ZEND_END_ARG_INFO()
 
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_table_exist, 0, 0, 1)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_table_exists, 0, 0, 1)
     ZEND_ARG_INFO(0, key)
 ZEND_END_ARG_INFO()
 
@@ -93,7 +93,7 @@ static PHP_METHOD(swoole_table, create);
 static PHP_METHOD(swoole_table, set);
 static PHP_METHOD(swoole_table, get);
 static PHP_METHOD(swoole_table, del);
-static PHP_METHOD(swoole_table, exist);
+static PHP_METHOD(swoole_table, exists);
 static PHP_METHOD(swoole_table, incr);
 static PHP_METHOD(swoole_table, decr);
 static PHP_METHOD(swoole_table, count);
@@ -126,7 +126,8 @@ static const zend_function_entry swoole_table_methods[] =
     PHP_ME(swoole_table, get,         arginfo_swoole_table_get, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_table, count,       arginfo_swoole_table_void, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_table, del,         arginfo_swoole_table_del, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_table, exist,       arginfo_swoole_table_exist, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_table, exists,      arginfo_swoole_table_exists, ZEND_ACC_PUBLIC)
+    PHP_MALIAS(swoole_table, exist, exists, arginfo_swoole_table_exists, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_table, incr,        arginfo_swoole_table_incr, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_table, decr,        arginfo_swoole_table_decr, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_table, getMemorySize,    arginfo_swoole_table_void, ZEND_ACC_PUBLIC)
@@ -405,18 +406,19 @@ static PHP_METHOD(swoole_table, set)
         }
         else if (col->type == SW_TABLE_STRING)
         {
-            convert_to_string(v);
-            swTableRow_set_value(row, col, Z_STRVAL_P(v), Z_STRLEN_P(v));
+            zend_string *str = zval_get_string(v);
+            swTableRow_set_value(row, col, ZSTR_VAL(str), ZSTR_LEN(str));
+            zend_string_release(str);
         }
         else if (col->type == SW_TABLE_FLOAT)
         {
-            convert_to_double(v);
-            swTableRow_set_value(row, col, &Z_DVAL_P(v), 0);
+            double _value = zval_get_double(v);
+            swTableRow_set_value(row, col, &_value, 0);
         }
         else
         {
-            convert_to_long(v);
-            swTableRow_set_value(row, col, &Z_LVAL_P(v), 0);
+            long _value = zval_get_long(v);
+            swTableRow_set_value(row, col, &_value, 0);
         }
     }
     (void) ktype;
@@ -479,8 +481,7 @@ static PHP_METHOD(swoole_table, incr)
         memcpy(&set_value, row->data + column->index, sizeof(set_value));
         if (incrby)
         {
-            convert_to_double(incrby);
-            set_value += Z_DVAL_P(incrby);
+            set_value += zval_get_double(incrby);
         }
         else
         {
@@ -495,8 +496,7 @@ static PHP_METHOD(swoole_table, incr)
         memcpy(&set_value, row->data + column->index, column->size);
         if (incrby)
         {
-            convert_to_long(incrby);
-            set_value += Z_LVAL_P(incrby);
+            set_value += zval_get_long(incrby);
         }
         else
         {
@@ -557,8 +557,7 @@ static PHP_METHOD(swoole_table, decr)
         memcpy(&set_value, row->data + column->index, sizeof(set_value));
         if (decrby)
         {
-            convert_to_double(decrby);
-            set_value -= Z_DVAL_P(decrby);
+            set_value -= zval_get_double(decrby);
         }
         else
         {
@@ -573,8 +572,7 @@ static PHP_METHOD(swoole_table, decr)
         memcpy(&set_value, row->data + column->index, column->size);
         if (decrby)
         {
-            convert_to_long(decrby);
-            set_value -= Z_LVAL_P(decrby);
+            set_value -= zval_get_long(decrby);
         }
         else
         {
@@ -642,28 +640,27 @@ static PHP_METHOD(swoole_table, offsetGet)
         RETURN_FALSE;
     }
 
-    zval *value;
-    SW_MAKE_STD_ZVAL(value);
+    zval value;
 
     swTableRow *row = swTableRow_get(table, key, keylen, &_rowlock);
     if (!row)
     {
-        array_init(value);
+        array_init(&value);
     }
     else
     {
-        php_swoole_table_row2array(table, row, value);
+        php_swoole_table_row2array(table, row, &value);
     }
     swTableRow_unlock(_rowlock);
 
     object_init_ex(return_value, swoole_table_row_ce_ptr);
-    zend_update_property(swoole_table_row_ce_ptr, return_value, ZEND_STRL("value"), value);
+    zend_update_property(swoole_table_row_ce_ptr, return_value, ZEND_STRL("value"), &value);
     zend_update_property_stringl(swoole_table_row_ce_ptr, return_value, ZEND_STRL("key"), key, keylen);
-    zval_ptr_dtor(value);
+    zval_ptr_dtor(&value);
     swoole_set_object(return_value, table);
 }
 
-static PHP_METHOD(swoole_table, exist)
+static PHP_METHOD(swoole_table, exists)
 {
     char *key;
     size_t keylen;
@@ -695,7 +692,7 @@ static PHP_METHOD(swoole_table, exist)
 
 static PHP_METHOD(swoole_table, offsetExists)
 {
-    ZEND_MN(swoole_table_exist)(INTERNAL_FUNCTION_PARAM_PASSTHRU);
+    ZEND_MN(swoole_table_exists)(INTERNAL_FUNCTION_PARAM_PASSTHRU);
 }
 
 static PHP_METHOD(swoole_table, del)
@@ -899,18 +896,19 @@ static PHP_METHOD(swoole_table_row, offsetSet)
     }
     if (col->type == SW_TABLE_STRING)
     {
-        convert_to_string(value);
-        swTableRow_set_value(row, col, Z_STRVAL_P(value), Z_STRLEN_P(value));
+        zend_string *str = zval_get_string(value);
+        swTableRow_set_value(row, col, ZSTR_VAL(str), ZSTR_LEN(str));
+        zend_string_release(str);
     }
     else if (col->type == SW_TABLE_FLOAT)
     {
-        convert_to_double(value);
-        swTableRow_set_value(row, col, &Z_DVAL_P(value), 0);
+        double _value = zval_get_double(value);
+        swTableRow_set_value(row, col, &_value, 0);
     }
     else
     {
-        convert_to_long(value);
-        swTableRow_set_value(row, col, &Z_LVAL_P(value), 0);
+        long _value = zval_get_long(value);
+        swTableRow_set_value(row, col, &_value, 0);
     }
     swTableRow_unlock(_rowlock);
 
