@@ -138,13 +138,12 @@ size_t trace_print_time(struct timeval *tv, char *timebuf, size_t timebuf_len)
     size_t len;
 
     len = strftime(timebuf, timebuf_len, "[%d-%b-%Y %H:%M:%S", localtime_r((const time_t *) &tv->tv_sec, &t));
-    len += snprintf(timebuf + len, timebuf_len - len, "] ");
+    len += sw_snprintf(timebuf + len, timebuf_len - len, "] ");
     return len;
 }
 
 static int trace_dump(swWorker *worker, FILE *slowlog)
 {
-    SWOOLE_GET_TSRMLS;
 
     pid_t traced_pid = worker->pid;
     int callers_limit = 100;
