@@ -14,7 +14,7 @@
   +----------------------------------------------------------------------+
 */
 
-#include "php_swoole.h"
+#include "php_swoole_cxx.h"
 #include "swoole_http.h"
 #include "swoole_coroutine.h"
 
@@ -33,6 +33,8 @@ extern "C"
 #include "base64.h"
 #include "thirdparty/swoole_http_parser.h"
 
+using namespace swoole;
+
 static zend_class_entry swoole_websocket_server_ce;
 zend_class_entry *swoole_websocket_server_ce_ptr;
 static zend_object_handlers swoole_websocket_server_handlers;
@@ -45,10 +47,8 @@ static zend_class_entry swoole_websocket_closeframe_ce;
 static zend_class_entry *swoole_websocket_closeframe_ce_ptr;
 static zend_object_handlers swoole_websocket_closeframe_handlers;
 
-static int websocket_handshake(swListenPort *, http_context *);
-
 static PHP_METHOD(swoole_websocket_server, push);
-static PHP_METHOD(swoole_websocket_server, exist);
+static PHP_METHOD(swoole_websocket_server, exists);
 static PHP_METHOD(swoole_websocket_server, isEstablished);
 static PHP_METHOD(swoole_websocket_server, pack);
 static PHP_METHOD(swoole_websocket_server, unpack);
@@ -80,7 +80,7 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_websocket_server_unpack, 0, 0, 1)
     ZEND_ARG_INFO(0, data)
 ZEND_END_ARG_INFO()
 
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_websocket_server_exist, 0, 0, 1)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_websocket_server_exists, 0, 0, 1)
     ZEND_ARG_INFO(0, fd)
 ZEND_END_ARG_INFO()
 
@@ -93,19 +93,20 @@ ZEND_END_ARG_INFO()
 
 const zend_function_entry swoole_websocket_server_methods[] =
 {
-    PHP_ME(swoole_websocket_server, push,           arginfo_swoole_websocket_server_push, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_websocket_server, disconnect,     arginfo_swoole_websocket_server_disconnect, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_websocket_server, exist,          arginfo_swoole_websocket_server_exist, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_websocket_server, isEstablished,  arginfo_swoole_websocket_server_isEstablished, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_websocket_server, pack,           arginfo_swoole_websocket_server_pack, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-    PHP_ME(swoole_websocket_server, unpack,         arginfo_swoole_websocket_server_unpack, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+    PHP_ME(swoole_websocket_server, push,              arginfo_swoole_websocket_server_push,          ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_websocket_server, disconnect,        arginfo_swoole_websocket_server_disconnect,    ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_websocket_server, exists,            arginfo_swoole_websocket_server_exists,         ZEND_ACC_PUBLIC)
+    PHP_MALIAS(swoole_websocket_server, exist, exists, arginfo_swoole_websocket_server_exists,         ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_websocket_server, isEstablished,     arginfo_swoole_websocket_server_isEstablished, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_websocket_server, pack,              arginfo_swoole_websocket_server_pack,          ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+    PHP_ME(swoole_websocket_server, unpack,            arginfo_swoole_websocket_server_unpack,        ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_FE_END
 };
 
 const zend_function_entry swoole_websocket_frame_methods[] =
 {
-    PHP_ME(swoole_websocket_frame, __toString,      arginfo_swoole_websocket_frame_void, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_websocket_server, pack,           arginfo_swoole_websocket_server_pack, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+    PHP_ME(swoole_websocket_frame, __toString,      arginfo_swoole_websocket_frame_void,    ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_websocket_server, pack,           arginfo_swoole_websocket_server_pack,   ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_websocket_server, unpack,         arginfo_swoole_websocket_server_unpack, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_FE_END
 };
@@ -167,15 +168,13 @@ int php_swoole_websocket_frame_pack(swString *buffer, zval *zdata, zend_bool opc
         zdata = NULL;
         if ((ztmp = sw_zend_read_property(swoole_websocket_frame_ce_ptr, zframe, ZEND_STRL("opcode"), 1)))
         {
-            convert_to_long(ztmp);
-            opcode = Z_LVAL_P(ztmp);
+            opcode = zval_get_long(ztmp);
         }
         if (opcode == WEBSOCKET_OPCODE_CLOSE)
         {
             if ((ztmp = sw_zend_read_property_not_null(swoole_websocket_frame_ce_ptr, zframe, ZEND_STRL("code"), 1)))
             {
-                convert_to_long(ztmp);
-                code = Z_LVAL_P(ztmp);
+                code = zval_get_long(ztmp);
             }
             if ((ztmp = sw_zend_read_property_not_null(swoole_websocket_frame_ce_ptr, zframe, ZEND_STRL("reason"), 1)))
             {
@@ -188,13 +187,11 @@ int php_swoole_websocket_frame_pack(swString *buffer, zval *zdata, zend_bool opc
         }
         if ((ztmp = sw_zend_read_property(swoole_websocket_frame_ce_ptr, zframe, ZEND_STRL("finish"), 1)))
         {
-            convert_to_boolean(ztmp);
-            fin = Z_BVAL_P(ztmp);
+            fin = zval_is_true(ztmp);
         }
         if ((ztmp = sw_zend_read_property(swoole_websocket_frame_ce_ptr, zframe, ZEND_STRL("mask"), 1)))
         {
-            convert_to_boolean(ztmp);
-            mask = Z_BVAL_P(ztmp);
+            mask = zval_is_true(ztmp);
         }
     }
     if (unlikely(opcode > SW_WEBSOCKET_OPCODE_MAX))
@@ -202,11 +199,12 @@ int php_swoole_websocket_frame_pack(swString *buffer, zval *zdata, zend_bool opc
         swoole_php_fatal_error(E_WARNING, "the maximum value of opcode is %d.", SW_WEBSOCKET_OPCODE_MAX);
         return SW_ERR;
     }
+    zend::string str_zdata;
     if (zdata && !ZVAL_IS_NULL(zdata))
     {
-        convert_to_string(zdata);
-        data = Z_STRVAL_P(zdata);
-        length = Z_STRLEN_P(zdata);
+        str_zdata = zdata;
+        data = str_zdata.val();
+        length = str_zdata.len();
     }
     switch(opcode)
     {
@@ -244,7 +242,7 @@ void swoole_websocket_onOpen(swServer *serv, http_context *ctx)
 
     if (SwooleG.enable_coroutine)
     {
-        if (sw_coro_create(fci_cache, 2, args) < 0)
+        if (PHPCoroutine::create(fci_cache, 2, args) < 0)
         {
             swoole_php_error(E_WARNING, "create onOpen coroutine error.");
             serv->close(serv, fd, 0);
@@ -276,9 +274,10 @@ void swoole_websocket_onRequest(http_context *ctx)
             "Server: " SW_HTTP_SERVER_SOFTWARE "\r\n\r\n"
             "<html><body><h2>HTTP 400 Bad Request</h2><hr><i>Powered by Swoole</i></body></html>";
 
-    swServer_tcp_send(SwooleG.serv, ctx->fd, (char *) bad_request, strlen(bad_request));
+    swServer *serv = SwooleG.serv;
+    serv->send(serv, ctx->fd, (char *) bad_request, strlen(bad_request));
     ctx->end = 1;
-    swServer_tcp_close(SwooleG.serv, ctx->fd, 0);
+    serv->close(serv, ctx->fd, 0);
     swoole_http_context_free(ctx);
 }
 
@@ -298,29 +297,26 @@ static int websocket_handshake(swServer *serv, swListenPort *port, http_context
 
     if (!(pData = zend_hash_str_find(ht, ZEND_STRL("sec-websocket-key"))))
     {
-        php_error_docref(NULL, E_WARNING, "header no sec-websocket-key");
+        swoole_php_fatal_error(E_WARNING, "header no sec-websocket-key");
         return SW_ERR;
     }
-    convert_to_string(pData);
 
+    zend::string str_pData(pData);
     swString_clear(swoole_http_buffer);
     swString_append_ptr(swoole_http_buffer, ZEND_STRL("HTTP/1.1 101 Switching Protocols\r\nUpgrade: websocket\r\nConnection: Upgrade\r\n"));
 
     int n;
-    char sec_websocket_accept[128];
-    memcpy(sec_websocket_accept, Z_STRVAL_P(pData), Z_STRLEN_P(pData));
-    memcpy(sec_websocket_accept + Z_STRLEN_P(pData), SW_WEBSOCKET_GUID, sizeof(SW_WEBSOCKET_GUID) - 1);
-
-    char sha1_str[20];
-    bzero(sha1_str, sizeof(sha1_str));
-    php_swoole_sha1(sec_websocket_accept, Z_STRLEN_P(pData) + sizeof(SW_WEBSOCKET_GUID) - 1, (unsigned char *) sha1_str);
-
-    char encoded_str[50];
-    bzero(encoded_str, sizeof(encoded_str));
-    n = swBase64_encode((unsigned char *) sha1_str, sizeof(sha1_str), encoded_str);
-
     char _buf[128];
-    n = snprintf(_buf, sizeof(_buf), "Sec-WebSocket-Accept: %*s\r\n", n, encoded_str);
+    char sha1_str[20];
+    char encoded_str[50];
+    // sec_websocket_accept
+    memcpy(_buf, str_pData.val(), str_pData.len());
+    memcpy(_buf + str_pData.len(), SW_WEBSOCKET_GUID, sizeof(SW_WEBSOCKET_GUID) - 1);
+    // sha1 sec_websocket_accept
+    php_swoole_sha1(_buf, str_pData.len() + sizeof(SW_WEBSOCKET_GUID) - 1, (unsigned char *) sha1_str);
+    // base64
+    n = swBase64_encode((unsigned char *) sha1_str, sizeof(sha1_str), encoded_str);
+    n = sw_snprintf(_buf, sizeof(_buf), "Sec-WebSocket-Accept: %.*s\r\n", n, encoded_str);
 
     swString_append_ptr(swoole_http_buffer, _buf, n);
     swString_append_ptr(swoole_http_buffer, ZEND_STRL("Sec-WebSocket-Version: " SW_WEBSOCKET_VERSION "\r\n"));
@@ -350,11 +346,9 @@ int swoole_websocket_onMessage(swServer *serv, swEventData *req)
     zend_bool finish = 0;
     zend_long opcode = 0;
 
-    zval *zdata;
-    SW_MAKE_STD_ZVAL(zdata);
-
+    zval zdata;
     char frame_header[2];
-    php_swoole_get_recv_data(zdata, req, frame_header, SW_WEBSOCKET_HEADER_LEN);
+    php_swoole_get_recv_data(&zdata, req, frame_header, SW_WEBSOCKET_HEADER_LEN);
 
     // frame info has already decoded in swWebSocket_dispatch_frame
     finish = frame_header[0] ? 1 : 0;
@@ -364,24 +358,23 @@ int swoole_websocket_onMessage(swServer *serv, swEventData *req)
     {
         if (!SwooleG.serv->listen_list->open_websocket_close_frame)
         {
-            zval_ptr_dtor(zdata);
+            zval_ptr_dtor(&zdata);
             return SW_OK;
         }
     }
 
-    zval *zframe;
-    SW_MAKE_STD_ZVAL(zframe);
-    php_swoole_websocket_construct_frame(zframe, opcode, Z_STRVAL_P(zdata), Z_STRLEN_P(zdata), finish);
-    zend_update_property_long(swoole_websocket_frame_ce_ptr, zframe, ZEND_STRL("fd"), fd);
+    zval zframe;
+    php_swoole_websocket_construct_frame(&zframe, opcode, Z_STRVAL(zdata), Z_STRLEN(zdata), finish);
+    zend_update_property_long(swoole_websocket_frame_ce_ptr, &zframe, ZEND_STRL("fd"), fd);
 
     zend_fcall_info_cache *fci_cache = php_swoole_server_get_fci_cache(serv, req->info.from_fd, SW_SERVER_CB_onMessage);
     zval args[2];
     args[0] = *(zval *) serv->ptr2; // zserver
-    args[1] = *zframe;
+    args[1] = zframe;
 
     if (SwooleG.enable_coroutine)
     {
-        if (sw_coro_create(fci_cache, 2, args) < 0)
+        if (PHPCoroutine::create(fci_cache, 2, args) < 0)
         {
             swoole_php_error(E_WARNING, "create onMessage coroutine error.");
             SwooleG.serv->factory.end(&SwooleG.serv->factory, fd);
@@ -397,8 +390,8 @@ int swoole_websocket_onMessage(swServer *serv, swEventData *req)
         zval_ptr_dtor(retval);
     }
 
-    zval_ptr_dtor(zdata);
-    zval_ptr_dtor(zframe);
+    zval_ptr_dtor(&zdata);
+    zval_ptr_dtor(&zframe);
 
     return SW_OK;
 }
@@ -409,7 +402,7 @@ int swoole_websocket_onHandshake(swServer *serv, swListenPort *port, http_contex
     int ret = websocket_handshake(serv, port, ctx);
     if (ret == SW_ERR)
     {
-        swServer_tcp_close(serv, fd, 1);
+        serv->close(serv, fd, 1);
     }
     else
     {
@@ -488,7 +481,7 @@ static sw_inline int swoole_websocket_server_push(swServer *serv, int fd, swStri
         return SW_ERR;
     }
 
-    int ret = swServer_tcp_send(SwooleG.serv, fd, buffer->str, buffer->length);
+    int ret = serv->send(serv, fd, buffer->str, buffer->length);
     if (ret < 0 && SwooleG.error == SW_ERROR_OUTPUT_BUFFER_OVERFLOW && serv->send_yield)
     {
         zval _return_value;
@@ -622,12 +615,12 @@ static PHP_METHOD(swoole_websocket_server, unpack)
     php_swoole_websocket_frame_unpack(&buffer, return_value);
 }
 
-static PHP_METHOD(swoole_websocket_server, exist)
+static PHP_METHOD(swoole_websocket_server, exists)
 {
     zend_long fd;
 
     swServer *serv = (swServer *) swoole_get_object(getThis());
-    if (serv->gs->start == 0)
+    if (unlikely(!serv->gs->start))
     {
         php_error_docref(NULL, E_WARNING, "the server is not running.");
         RETURN_FALSE;
@@ -671,7 +664,7 @@ static PHP_METHOD(swoole_websocket_server, isEstablished)
     zend_long fd;
 
     swServer *serv = (swServer *) swoole_get_object(getThis());
-    if (serv->gs->start == 0)
+    if (unlikely(!serv->gs->start))
     {
         php_error_docref(NULL, E_WARNING, "the server is not running.");
         RETURN_FALSE;
