@@ -174,6 +174,17 @@ $content = str_replace($dir_tag, $dir_tag . $space . implode("{$space}", $file_l
 if (!$success) {
     swoole_error('Replace new content failed!');
 }
+date_default_timezone_set('Asia/Shanghai');
+$date_tag = date('Y-m-d');
+$content = preg_replace('/(<date\>)\d+?-\d+?-\d+?(<\/date>)/', '${1}' . $date_tag . '${2}', $content, $success);
+if (!$success) {
+    swoole_error('Replace date tag failed!');
+}
+$time_tag = date('H', time() + 3600) . ':00:00';
+$content = preg_replace('/(<time\>)\d+?:\d+?:\d+?(<\/time>)/', '${1}' . $time_tag . '${2}', $content, $success);
+if (!$success) {
+    swoole_error('Replace time tag failed!');
+}
 if (!file_put_contents(__DIR__ . '/../package.xml', $content)) {
     swoole_error('Output package successful!');
 }
