@@ -2,9 +2,8 @@
 __CURRENT__=`pwd`
 __DIR__=$(cd "$(dirname "$0")";pwd)
 
-# show dir info
-cd ${__DIR__} && pwd
-ls -al / && echo ""
+# enter the dir
+cd ${__DIR__}
 
 # show system info
 date && echo ""
@@ -19,11 +18,5 @@ php -v && echo ""
 # swoole info
 php --ri swoole
 
-# alpine
-if [ "`apk 2>&1 | grep apk-tools`"x != ""x ]; then
-  echo -e "\n😪 skip alpine\n"
-  exit 0
-fi
-
 # run unit tests
 ./run-tests.sh
